location = {
    0: {'negeri': 'JOHOR', 'parlimen': 'SEGAMAT', 'dun': 'BULOH KASAP'},
    1: {'negeri': 'JOHOR', 'parlimen': 'SEGAMAT', 'dun': 'JEMENTAH'},
    2: {'negeri': 'JOHOR', 'parlimen': 'SEKIJANG', 'dun': 'PEMANIS'},
    3: {'negeri': 'JOHOR', 'parlimen': 'SEKIJANG', 'dun': 'KEMELAH'},
    4: {'negeri': 'JOHOR', 'parlimen': 'LABIS', 'dun': 'TENANG'},
    5: {'negeri': 'JOHOR', 'parlimen': 'LABIS', 'dun': 'BEKOK'},
    6: {'negeri': 'JOHOR', 'parlimen': 'PAGOH', 'dun': 'BUKIT SERAMPANG'},
    7: {'negeri': 'JOHOR', 'parlimen': 'PAGOH', 'dun': 'JORAK'},
    8: {'negeri': 'JOHOR', 'parlimen': 'LEDANG', 'dun': 'GAMBIR'},
    9: {'negeri': 'JOHOR', 'parlimen': 'LEDANG', 'dun': 'TANGKAK'},
    10: {'negeri': 'JOHOR', 'parlimen': 'LEDANG', 'dun': 'SEROM'},
    11: {'negeri': 'JOHOR', 'parlimen': 'BAKRI', 'dun': 'BENTAYAN'},
    12: {'negeri': 'JOHOR', 'parlimen': 'BAKRI', 'dun': 'SUNGAI ABONG'},
    13: {'negeri': 'JOHOR', 'parlimen': 'BAKRI', 'dun': 'BUKIT NANING'},
    14: {'negeri': 'JOHOR', 'parlimen': 'MUAR', 'dun': 'MAHARANI'},
    15: {'negeri': 'JOHOR', 'parlimen': 'MUAR', 'dun': 'SUNGAI BALANG'},
    16: {'negeri': 'JOHOR', 'parlimen': 'PARIT SULONG', 'dun': 'SEMERAH'},
    17: {'negeri': 'JOHOR', 'parlimen': 'PARIT SULONG', 'dun': 'SRI MEDAN'},
    18: {'negeri': 'JOHOR', 'parlimen': 'AYER HITAM', 'dun': 'YONG PENG'},
    19: {'negeri': 'JOHOR', 'parlimen': 'AYER HITAM', 'dun': 'SEMARANG'},
    20: {'negeri': 'JOHOR', 'parlimen': 'SRI GADING', 'dun': 'PARIT YAANI'},
    21: {'negeri': 'JOHOR', 'parlimen': 'SRI GADING', 'dun': 'PARIT RAJA'},
    22: {'negeri': 'JOHOR', 'parlimen': 'BATU PAHAT', 'dun': 'PENGGARAM'},
    23: {'negeri': 'JOHOR', 'parlimen': 'BATU PAHAT', 'dun': 'SENGGARANG'},
    24: {'negeri': 'JOHOR', 'parlimen': 'BATU PAHAT', 'dun': 'RENGIT'},
    25: {'negeri': 'JOHOR', 'parlimen': 'SIMPANG RENGGAM', 'dun': 'MACHAP'},
    26: {
        'negeri': 'JOHOR',
        'parlimen': 'SIMPANG RENGGAM',
        'dun': 'LAYANG-LAYANG',
    },
    27: {'negeri': 'JOHOR', 'parlimen': 'KLUANG', 'dun': 'MENGKIBOL'},
    28: {'negeri': 'JOHOR', 'parlimen': 'KLUANG', 'dun': 'MAHKOTA'},
    29: {'negeri': 'JOHOR', 'parlimen': 'SEMBRONG', 'dun': 'PALOH'},
    30: {'negeri': 'JOHOR', 'parlimen': 'SEMBRONG', 'dun': 'KAHANG'},
    31: {'negeri': 'JOHOR', 'parlimen': 'MERSING', 'dun': 'ENDAU'},
    32: {'negeri': 'JOHOR', 'parlimen': 'MERSING', 'dun': 'TENGGAROH'},
    33: {'negeri': 'JOHOR', 'parlimen': 'TENGGARA', 'dun': 'PANTI'},
    34: {'negeri': 'JOHOR', 'parlimen': 'TENGGARA', 'dun': 'PASIR RAJA'},
    35: {'negeri': 'JOHOR', 'parlimen': 'KOTA TINGGI', 'dun': 'SEDILI'},
    36: {'negeri': 'JOHOR', 'parlimen': 'KOTA TINGGI', 'dun': 'JOHOR LAMA'},
    37: {'negeri': 'JOHOR', 'parlimen': 'PENGERANG', 'dun': 'PENAWAR'},
    38: {'negeri': 'JOHOR', 'parlimen': 'PENGERANG', 'dun': 'TANJONG SURAT'},
    39: {'negeri': 'JOHOR', 'parlimen': 'TEBRAU', 'dun': 'TIRAM'},
    40: {'negeri': 'JOHOR', 'parlimen': 'TEBRAU', 'dun': 'PUTERI WANGSA'},
    41: {'negeri': 'JOHOR', 'parlimen': 'PASIR GUDANG', 'dun': 'JOHOR JAYA'},
    42: {'negeri': 'JOHOR', 'parlimen': 'PASIR GUDANG', 'dun': 'PERMAS'},
    43: {'negeri': 'JOHOR', 'parlimen': 'JOHOR BAHRU', 'dun': 'TANJONG PUTERI'},
    44: {'negeri': 'JOHOR', 'parlimen': 'JOHOR BAHRU', 'dun': 'STULANG'},
    45: {'negeri': 'JOHOR', 'parlimen': 'PULAI', 'dun': 'PENGKALAN RINTING'},
    46: {'negeri': 'JOHOR', 'parlimen': 'PULAI', 'dun': 'KEMPAS'},
    47: {'negeri': 'JOHOR', 'parlimen': 'GELANG PATAH', 'dun': 'SKUDAI'},
    48: {'negeri': 'JOHOR', 'parlimen': 'GELANG PATAH', 'dun': 'NUSA JAYA'},
    49: {'negeri': 'JOHOR', 'parlimen': 'KULAI', 'dun': 'BUKIT PERMAI'},
    50: {'negeri': 'JOHOR', 'parlimen': 'KULAI', 'dun': 'BUKIT BATU'},
    51: {'negeri': 'JOHOR', 'parlimen': 'KULAI', 'dun': 'SENAI'},
    52: {'negeri': 'JOHOR', 'parlimen': 'PONTIAN', 'dun': 'BENUT'},
    53: {'negeri': 'JOHOR', 'parlimen': 'PONTIAN', 'dun': 'PULAI SEBATANG'},
    54: {'negeri': 'JOHOR', 'parlimen': 'TANJONG PIAI', 'dun': 'PEKAN NENAS'},
    55: {'negeri': 'JOHOR', 'parlimen': 'TANJONG PIAI', 'dun': 'KUKUP'},
    56: {'negeri': 'KEDAH', 'parlimen': 'LANGKAWI', 'dun': 'AYER HANGAT'},
    57: {'negeri': 'KEDAH', 'parlimen': 'LANGKAWI', 'dun': 'KUAH'},
    58: {'negeri': 'KEDAH', 'parlimen': 'JERLUN', 'dun': 'KOTA SIPUTEH'},
    59: {'negeri': 'KEDAH', 'parlimen': 'JERLUN', 'dun': 'AYER HITAM'},
    60: {
        'negeri': 'KEDAH',
        'parlimen': 'KUBANG PASU',
        'dun': 'BUKIT KAYU HITAM',
    },
    61: {'negeri': 'KEDAH', 'parlimen': 'KUBANG PASU', 'dun': 'JITRA'},
    62: {'negeri': 'KEDAH', 'parlimen': 'PADANG TERAP', 'dun': 'KUALA NERANG'},
    63: {'negeri': 'KEDAH', 'parlimen': 'PADANG TERAP', 'dun': 'PEDU'},
    64: {'negeri': 'KEDAH', 'parlimen': 'POKOK SENA', 'dun': 'BUKIT LADA'},
    65: {'negeri': 'KEDAH', 'parlimen': 'POKOK SENA', 'dun': 'BUKIT PINANG'},
    66: {'negeri': 'KEDAH', 'parlimen': 'POKOK SENA', 'dun': 'DERGA'},
    67: {'negeri': 'KEDAH', 'parlimen': 'ALOR STAR', 'dun': 'BAKAR BATA'},
    68: {'negeri': 'KEDAH', 'parlimen': 'ALOR STAR', 'dun': 'KOTA DARUL AMAN'},
    69: {'negeri': 'KEDAH', 'parlimen': 'ALOR STAR', 'dun': 'ALOR MENGKUDU'},
    70: {'negeri': 'KEDAH', 'parlimen': 'KUALA KEDAH', 'dun': 'ANAK BUKIT'},
    71: {'negeri': 'KEDAH', 'parlimen': 'KUALA KEDAH', 'dun': 'KUBANG ROTAN'},
    72: {
        'negeri': 'KEDAH',
        'parlimen': 'KUALA KEDAH',
        'dun': 'PENGKALAN KUNDOR',
    },
    73: {'negeri': 'KEDAH', 'parlimen': 'PENDANG', 'dun': 'TOKAI'},
    74: {'negeri': 'KEDAH', 'parlimen': 'PENDANG', 'dun': 'SUNGAI TIANG'},
    75: {'negeri': 'KEDAH', 'parlimen': 'JERAI', 'dun': 'SUNGAI LIMAU'},
    76: {'negeri': 'KEDAH', 'parlimen': 'JERAI', 'dun': 'GUAR CHEMPEDAK'},
    77: {'negeri': 'KEDAH', 'parlimen': 'JERAI', 'dun': 'GURUN'},
    78: {'negeri': 'KEDAH', 'parlimen': 'SIK', 'dun': 'BELANTEK'},
    79: {'negeri': 'KEDAH', 'parlimen': 'SIK', 'dun': 'JENERI'},
    80: {'negeri': 'KEDAH', 'parlimen': 'MERBOK', 'dun': 'BUKIT SELAMBAU'},
    81: {'negeri': 'KEDAH', 'parlimen': 'MERBOK', 'dun': 'TANJONG DAWAI'},
    82: {
        'negeri': 'KEDAH',
        'parlimen': 'SUNGAI PETANI',
        'dun': 'PANTAI MERDEKA',
    },
    83: {'negeri': 'KEDAH', 'parlimen': 'SUNGAI PETANI', 'dun': 'BAKAR ARANG'},
    84: {'negeri': 'KEDAH', 'parlimen': 'SUNGAI PETANI', 'dun': 'SIDAM'},
    85: {'negeri': 'KEDAH', 'parlimen': 'BALING', 'dun': 'BAYU'},
    86: {'negeri': 'KEDAH', 'parlimen': 'BALING', 'dun': 'KUPANG'},
    87: {'negeri': 'KEDAH', 'parlimen': 'BALING', 'dun': 'KUALA KETIL'},
    88: {'negeri': 'KEDAH', 'parlimen': 'PADANG SERAI', 'dun': 'MERBAU PULAS'},
    89: {'negeri': 'KEDAH', 'parlimen': 'PADANG SERAI', 'dun': 'LUNAS'},
    90: {'negeri': 'KEDAH', 'parlimen': 'KULIM-BANDAR BAHARU', 'dun': 'KULIM'},
    91: {
        'negeri': 'KEDAH',
        'parlimen': 'KULIM-BANDAR BAHARU',
        'dun': 'BANDAR BAHARU',
    },
    92: {'negeri': 'KELANTAN', 'parlimen': 'TUMPAT', 'dun': 'PENGKALAN KUBOR'},
    93: {'negeri': 'KELANTAN', 'parlimen': 'TUMPAT', 'dun': 'KELABORAN'},
    94: {'negeri': 'KELANTAN', 'parlimen': 'TUMPAT', 'dun': 'PASIR PEKAN'},
    95: {'negeri': 'KELANTAN', 'parlimen': 'TUMPAT', 'dun': 'WAKAF BHARU'},
    96: {'negeri': 'KELANTAN', 'parlimen': 'PENGKALAN CHEPA', 'dun': 'KIJANG'},
    97: {
        'negeri': 'KELANTAN',
        'parlimen': 'PENGKALAN CHEPA',
        'dun': 'CHEMPAKA',
    },
    98: {'negeri': 'KELANTAN', 'parlimen': 'PENGKALAN CHEPA', 'dun': 'PANCHOR'},
    99: {'negeri': 'KELANTAN', 'parlimen': 'KOTA BHARU', 'dun': 'TANJONG MAS'},
    100: {'negeri': 'KELANTAN', 'parlimen': 'KOTA BHARU', 'dun': 'KOTA LAMA'},
    101: {
        'negeri': 'KELANTAN',
        'parlimen': 'KOTA BHARU',
        'dun': 'BUNUT PAYONG',
    },
    102: {'negeri': 'KELANTAN', 'parlimen': 'PASIR MAS', 'dun': 'TENDONG'},
    103: {
        'negeri': 'KELANTAN',
        'parlimen': 'PASIR MAS',
        'dun': 'PENGKALAN PASIR',
    },
    104: {'negeri': 'KELANTAN', 'parlimen': 'PASIR MAS', 'dun': 'CHETOK'},
    105: {'negeri': 'KELANTAN', 'parlimen': 'RANTAU PANJANG', 'dun': 'MERANTI'},
    106: {
        'negeri': 'KELANTAN',
        'parlimen': 'RANTAU PANJANG',
        'dun': 'GUAL PERIOK',
    },
    107: {
        'negeri': 'KELANTAN',
        'parlimen': 'RANTAU PANJANG',
        'dun': 'BUKIT TUKU',
    },
    108: {'negeri': 'KELANTAN', 'parlimen': 'KUBANG KERIAN', 'dun': 'SALOR'},
    109: {
        'negeri': 'KELANTAN',
        'parlimen': 'KUBANG KERIAN',
        'dun': 'PASIR TUMBOH',
    },
    110: {'negeri': 'KELANTAN', 'parlimen': 'KUBANG KERIAN', 'dun': 'DEMIT'},
    111: {'negeri': 'KELANTAN', 'parlimen': 'BACHOK', 'dun': 'TAWANG'},
    112: {'negeri': 'KELANTAN', 'parlimen': 'BACHOK', 'dun': 'PERUPOK'},
    113: {'negeri': 'KELANTAN', 'parlimen': 'BACHOK', 'dun': 'JELAWAT'},
    114: {'negeri': 'KELANTAN', 'parlimen': 'KETEREH', 'dun': 'MELOR'},
    115: {'negeri': 'KELANTAN', 'parlimen': 'KETEREH', 'dun': 'KADOK'},
    116: {'negeri': 'KELANTAN', 'parlimen': 'KETEREH', 'dun': 'KOK LANAS'},
    117: {
        'negeri': 'KELANTAN',
        'parlimen': 'TANAH MERAH',
        'dun': 'BUKIT PANAU',
    },
    118: {'negeri': 'KELANTAN', 'parlimen': 'TANAH MERAH', 'dun': 'GUAL IPOH'},
    119: {'negeri': 'KELANTAN', 'parlimen': 'TANAH MERAH', 'dun': 'KEMAHANG'},
    120: {'negeri': 'KELANTAN', 'parlimen': 'PASIR PUTEH', 'dun': 'SELISING'},
    121: {'negeri': 'KELANTAN', 'parlimen': 'PASIR PUTEH', 'dun': 'LIMBONGAN'},
    122: {'negeri': 'KELANTAN', 'parlimen': 'PASIR PUTEH', 'dun': 'SEMERAK'},
    123: {'negeri': 'KELANTAN', 'parlimen': 'PASIR PUTEH', 'dun': 'GAAL'},
    124: {'negeri': 'KELANTAN', 'parlimen': 'MACHANG', 'dun': 'PULAI CHONDONG'},
    125: {'negeri': 'KELANTAN', 'parlimen': 'MACHANG', 'dun': 'TEMANGAN'},
    126: {'negeri': 'KELANTAN', 'parlimen': 'MACHANG', 'dun': 'KEMUNING'},
    127: {'negeri': 'KELANTAN', 'parlimen': 'JELI', 'dun': 'BUKIT BUNGA'},
    128: {'negeri': 'KELANTAN', 'parlimen': 'JELI', 'dun': 'AIR LANAS'},
    129: {'negeri': 'KELANTAN', 'parlimen': 'JELI', 'dun': 'KUALA BALAH'},
    130: {'negeri': 'KELANTAN', 'parlimen': 'KUALA KRAI', 'dun': 'MENGKEBANG'},
    131: {'negeri': 'KELANTAN', 'parlimen': 'KUALA KRAI', 'dun': 'GUCHIL'},
    132: {'negeri': 'KELANTAN', 'parlimen': 'KUALA KRAI', 'dun': 'MANEK URAI'},
    133: {'negeri': 'KELANTAN', 'parlimen': 'KUALA KRAI', 'dun': 'DABONG'},
    134: {'negeri': 'KELANTAN', 'parlimen': 'GUA MUSANG', 'dun': 'NENGGIRI'},
    135: {'negeri': 'KELANTAN', 'parlimen': 'GUA MUSANG', 'dun': 'PALOH'},
    136: {'negeri': 'KELANTAN', 'parlimen': 'GUA MUSANG', 'dun': 'GALAS'},
    137: {
        'negeri': 'MELAKA',
        'parlimen': 'MASJID TANAH',
        'dun': 'KUALA LINGGI',
    },
    138: {
        'negeri': 'MELAKA',
        'parlimen': 'MASJID TANAH',
        'dun': 'TANJUNG BIDARA',
    },
    139: {'negeri': 'MELAKA', 'parlimen': 'MASJID TANAH', 'dun': 'AYER LIMAU'},
    140: {'negeri': 'MELAKA', 'parlimen': 'MASJID TANAH', 'dun': 'LENDU'},
    141: {
        'negeri': 'MELAKA',
        'parlimen': 'MASJID TANAH',
        'dun': 'TABOH NANING',
    },
    142: {'negeri': 'MELAKA', 'parlimen': 'ALOR GAJAH', 'dun': 'REMBIA'},
    143: {'negeri': 'MELAKA', 'parlimen': 'ALOR GAJAH', 'dun': 'GADEK'},
    144: {'negeri': 'MELAKA', 'parlimen': 'ALOR GAJAH', 'dun': 'MACHAP'},
    145: {
        'negeri': 'MELAKA',
        'parlimen': 'ALOR GAJAH',
        'dun': 'DURIAN TUNGGAL',
    },
    146: {'negeri': 'MELAKA', 'parlimen': 'ALOR GAJAH', 'dun': 'ASAHAN'},
    147: {'negeri': 'MELAKA', 'parlimen': 'TANGGA BATU', 'dun': 'SUNGAI UDANG'},
    148: {
        'negeri': 'MELAKA',
        'parlimen': 'TANGGA BATU',
        'dun': 'PANTAI KUNDOR',
    },
    149: {'negeri': 'MELAKA', 'parlimen': 'TANGGA BATU', 'dun': 'PAYA RUMPUT'},
    150: {'negeri': 'MELAKA', 'parlimen': 'TANGGA BATU', 'dun': 'KELEBANG'},
    151: {'negeri': 'MELAKA', 'parlimen': 'BUKIT KATIL', 'dun': 'BACHANG'},
    152: {'negeri': 'MELAKA', 'parlimen': 'BUKIT KATIL', 'dun': 'AYER KEROH'},
    153: {'negeri': 'MELAKA', 'parlimen': 'BUKIT KATIL', 'dun': 'BUKIT BARU'},
    154: {'negeri': 'MELAKA', 'parlimen': 'BUKIT KATIL', 'dun': 'AYER MOLEK'},
    155: {'negeri': 'MELAKA', 'parlimen': 'KOTA MELAKA', 'dun': 'KESIDANG'},
    156: {
        'negeri': 'MELAKA',
        'parlimen': 'KOTA MELAKA',
        'dun': 'KOTA LAKSAMANA',
    },
    157: {'negeri': 'MELAKA', 'parlimen': 'KOTA MELAKA', 'dun': 'DUYONG'},
    158: {'negeri': 'MELAKA', 'parlimen': 'KOTA MELAKA', 'dun': 'BANDAR HILIR'},
    159: {'negeri': 'MELAKA', 'parlimen': 'KOTA MELAKA', 'dun': 'TELOK MAS'},
    160: {'negeri': 'MELAKA', 'parlimen': 'JASIN', 'dun': 'BEMBAN'},
    161: {'negeri': 'MELAKA', 'parlimen': 'JASIN', 'dun': 'RIM'},
    162: {'negeri': 'MELAKA', 'parlimen': 'JASIN', 'dun': 'SERKAM'},
    163: {'negeri': 'MELAKA', 'parlimen': 'JASIN', 'dun': 'MERLIMAU'},
    164: {'negeri': 'MELAKA', 'parlimen': 'JASIN', 'dun': 'SUNGAI RAMBAI'},
    165: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'JELEBU', 'dun': 'CHENNAH'},
    166: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'JELEBU', 'dun': 'PERTANG'},
    167: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'JELEBU',
        'dun': 'SUNGAI LUI',
    },
    168: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'JELEBU', 'dun': 'KLAWANG'},
    169: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'JEMPOL', 'dun': 'SERTING'},
    170: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'JEMPOL', 'dun': 'PALONG'},
    171: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'JEMPOL',
        'dun': 'JERAM PADANG',
    },
    172: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'JEMPOL', 'dun': 'BAHAU'},
    173: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'SEREMBAN',
        'dun': 'LENGGENG',
    },
    174: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'SEREMBAN', 'dun': 'NILAI'},
    175: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'SEREMBAN', 'dun': 'LOBAK'},
    176: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'SEREMBAN',
        'dun': 'TEMIANG',
    },
    177: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'SEREMBAN',
        'dun': 'SIKAMAT',
    },
    178: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'SEREMBAN',
        'dun': 'AMPANGAN',
    },
    179: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'KUALA PILAH',
        'dun': 'JUASSEH',
    },
    180: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'KUALA PILAH',
        'dun': 'SERI MENANTI',
    },
    181: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'KUALA PILAH',
        'dun': 'SENALING',
    },
    182: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'KUALA PILAH',
        'dun': 'PILAH',
    },
    183: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'KUALA PILAH',
        'dun': 'JOHOL',
    },
    184: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'RASAH', 'dun': 'LABU'},
    185: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'RASAH',
        'dun': 'BUKIT KEPAYANG',
    },
    186: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'RASAH', 'dun': 'RAHANG'},
    187: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'RASAH', 'dun': 'MAMBAU'},
    188: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'RASAH', 'dun': 'SENAWANG'},
    189: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'REMBAU', 'dun': 'PAROI'},
    190: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'REMBAU', 'dun': 'CHEMBONG'},
    191: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'REMBAU', 'dun': 'RANTAU'},
    192: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'REMBAU', 'dun': 'KOTA'},
    193: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'TELOK KEMANG',
        'dun': 'CHUAH',
    },
    194: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'TELOK KEMANG',
        'dun': 'LUKUT',
    },
    195: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'TELOK KEMANG',
        'dun': 'BAGAN PINANG',
    },
    196: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'TELOK KEMANG',
        'dun': 'LINGGI',
    },
    197: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'TELOK KEMANG',
        'dun': 'PORT DICKSON',
    },
    198: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'TAMPIN', 'dun': 'GEMAS'},
    199: {
        'negeri': 'NEGERI SEMBILAN',
        'parlimen': 'TAMPIN',
        'dun': 'GEMENCHEH',
    },
    200: {'negeri': 'NEGERI SEMBILAN', 'parlimen': 'TAMPIN', 'dun': 'REPAH'},
    201: {
        'negeri': 'PAHANG',
        'parlimen': 'CAMERON HIGHLANDS',
        'dun': 'TANAH RATA',
    },
    202: {'negeri': 'PAHANG', 'parlimen': 'CAMERON HIGHLANDS', 'dun': 'JELAI'},
    203: {'negeri': 'PAHANG', 'parlimen': 'LIPIS', 'dun': 'PADANG TENGKU'},
    204: {'negeri': 'PAHANG', 'parlimen': 'LIPIS', 'dun': 'CHEKA'},
    205: {'negeri': 'PAHANG', 'parlimen': 'LIPIS', 'dun': 'BENTA'},
    206: {'negeri': 'PAHANG', 'parlimen': 'RAUB', 'dun': 'BATU TALAM'},
    207: {'negeri': 'PAHANG', 'parlimen': 'RAUB', 'dun': 'TRAS'},
    208: {'negeri': 'PAHANG', 'parlimen': 'RAUB', 'dun': 'DONG'},
    209: {'negeri': 'PAHANG', 'parlimen': 'JERANTUT', 'dun': 'TAHAN'},
    210: {'negeri': 'PAHANG', 'parlimen': 'JERANTUT', 'dun': 'DAMAK'},
    211: {'negeri': 'PAHANG', 'parlimen': 'JERANTUT', 'dun': 'PULAU TAWAR'},
    212: {'negeri': 'PAHANG', 'parlimen': 'INDERA MAHKOTA', 'dun': 'BESERAH'},
    213: {'negeri': 'PAHANG', 'parlimen': 'INDERA MAHKOTA', 'dun': 'SEMAMBU'},
    214: {'negeri': 'PAHANG', 'parlimen': 'KUANTAN', 'dun': 'TERUNTUM'},
    215: {'negeri': 'PAHANG', 'parlimen': 'KUANTAN', 'dun': 'TANJUNG LUMPUR'},
    216: {'negeri': 'PAHANG', 'parlimen': 'KUANTAN', 'dun': 'INDERAPURA'},
    217: {
        'negeri': 'PAHANG',
        'parlimen': 'PAYA BESAR',
        'dun': 'SUNGAI LEMBING',
    },
    218: {'negeri': 'PAHANG', 'parlimen': 'PAYA BESAR', 'dun': 'LEPAR'},
    219: {'negeri': 'PAHANG', 'parlimen': 'PAYA BESAR', 'dun': 'PANCHING'},
    220: {'negeri': 'PAHANG', 'parlimen': 'PEKAN', 'dun': 'PULAU MANIS'},
    221: {'negeri': 'PAHANG', 'parlimen': 'PEKAN', 'dun': 'PERAMU JAYA'},
    222: {'negeri': 'PAHANG', 'parlimen': 'PEKAN', 'dun': 'BEBAR'},
    223: {'negeri': 'PAHANG', 'parlimen': 'PEKAN', 'dun': 'CHINI'},
    224: {'negeri': 'PAHANG', 'parlimen': 'MARAN', 'dun': 'LUIT'},
    225: {'negeri': 'PAHANG', 'parlimen': 'MARAN', 'dun': 'KUALA SENTUL'},
    226: {'negeri': 'PAHANG', 'parlimen': 'MARAN', 'dun': 'CHENOR'},
    227: {'negeri': 'PAHANG', 'parlimen': 'KUALA KRAU', 'dun': 'JENDERAK'},
    228: {'negeri': 'PAHANG', 'parlimen': 'KUALA KRAU', 'dun': 'KERDAU'},
    229: {'negeri': 'PAHANG', 'parlimen': 'KUALA KRAU', 'dun': 'JENGKA'},
    230: {'negeri': 'PAHANG', 'parlimen': 'TEMERLOH', 'dun': 'MENTAKAB'},
    231: {'negeri': 'PAHANG', 'parlimen': 'TEMERLOH', 'dun': 'LANCHANG'},
    232: {'negeri': 'PAHANG', 'parlimen': 'TEMERLOH', 'dun': 'KUALA SEMANTAN'},
    233: {'negeri': 'PAHANG', 'parlimen': 'BENTONG', 'dun': 'BILUT'},
    234: {'negeri': 'PAHANG', 'parlimen': 'BENTONG', 'dun': 'KETARI'},
    235: {'negeri': 'PAHANG', 'parlimen': 'BENTONG', 'dun': 'SABAI'},
    236: {'negeri': 'PAHANG', 'parlimen': 'BENTONG', 'dun': 'PELANGAI'},
    237: {'negeri': 'PAHANG', 'parlimen': 'BERA', 'dun': 'GUAI'},
    238: {'negeri': 'PAHANG', 'parlimen': 'BERA', 'dun': 'TRIANG'},
    239: {'negeri': 'PAHANG', 'parlimen': 'BERA', 'dun': 'KEMAYAN'},
    240: {'negeri': 'PAHANG', 'parlimen': 'ROMPIN', 'dun': 'BUKIT IBAM'},
    241: {'negeri': 'PAHANG', 'parlimen': 'ROMPIN', 'dun': 'MUADZAM SHAH'},
    242: {'negeri': 'PAHANG', 'parlimen': 'ROMPIN', 'dun': 'TIOMAN'},
    243: {'negeri': 'PERAK', 'parlimen': 'GERIK', 'dun': 'PENGKALAN HULU'},
    244: {'negeri': 'PERAK', 'parlimen': 'GERIK', 'dun': 'TEMENGOR'},
    245: {'negeri': 'PERAK', 'parlimen': 'LENGGONG', 'dun': 'KENERING'},
    246: {'negeri': 'PERAK', 'parlimen': 'LENGGONG', 'dun': 'KOTA TAMPAN'},
    247: {'negeri': 'PERAK', 'parlimen': 'LARUT', 'dun': 'SELAMA'},
    248: {'negeri': 'PERAK', 'parlimen': 'LARUT', 'dun': 'KUBU GAJAH'},
    249: {'negeri': 'PERAK', 'parlimen': 'LARUT', 'dun': 'BATU KURAU'},
    250: {'negeri': 'PERAK', 'parlimen': 'PARIT BUNTAR', 'dun': 'TITI SERONG'},
    251: {'negeri': 'PERAK', 'parlimen': 'PARIT BUNTAR', 'dun': 'KUALA KURAU'},
    252: {'negeri': 'PERAK', 'parlimen': 'BAGAN SERAI', 'dun': 'ALOR PONGSU'},
    253: {
        'negeri': 'PERAK',
        'parlimen': 'BAGAN SERAI',
        'dun': 'GUNONG SEMANGGOL',
    },
    254: {'negeri': 'PERAK', 'parlimen': 'BAGAN SERAI', 'dun': 'SELINSING'},
    255: {
        'negeri': 'PERAK',
        'parlimen': 'BUKIT GANTANG',
        'dun': 'KUALA SAPETANG',
    },
    256: {
        'negeri': 'PERAK',
        'parlimen': 'BUKIT GANTANG',
        'dun': 'CHANGKAT JERING',
    },
    257: {'negeri': 'PERAK', 'parlimen': 'BUKIT GANTANG', 'dun': 'TRONG'},
    258: {'negeri': 'PERAK', 'parlimen': 'TAIPING', 'dun': 'KAMUNTING'},
    259: {'negeri': 'PERAK', 'parlimen': 'TAIPING', 'dun': 'POKOK ASSAM'},
    260: {'negeri': 'PERAK', 'parlimen': 'TAIPING', 'dun': 'AULONG'},
    261: {'negeri': 'PERAK', 'parlimen': 'PADANG RENGAS', 'dun': 'CHENDEROH'},
    262: {
        'negeri': 'PERAK',
        'parlimen': 'PADANG RENGAS',
        'dun': 'LUBOK MERBAU',
    },
    263: {'negeri': 'PERAK', 'parlimen': 'SUNGAI SIPUT', 'dun': 'LINTANG'},
    264: {'negeri': 'PERAK', 'parlimen': 'SUNGAI SIPUT', 'dun': 'JALONG'},
    265: {'negeri': 'PERAK', 'parlimen': 'TAMBUN', 'dun': 'MANJOI'},
    266: {'negeri': 'PERAK', 'parlimen': 'TAMBUN', 'dun': 'HULU KINTA'},
    267: {'negeri': 'PERAK', 'parlimen': 'IPOH TIMOR', 'dun': 'CANNING'},
    268: {'negeri': 'PERAK', 'parlimen': 'IPOH TIMOR', 'dun': 'TEBING TINGGI'},
    269: {'negeri': 'PERAK', 'parlimen': 'IPOH TIMOR', 'dun': 'PASIR PINJI'},
    270: {'negeri': 'PERAK', 'parlimen': 'IPOH BARAT', 'dun': 'BERCHAM'},
    271: {'negeri': 'PERAK', 'parlimen': 'IPOH BARAT', 'dun': 'KEPAYANG'},
    272: {'negeri': 'PERAK', 'parlimen': 'IPOH BARAT', 'dun': 'BUNTONG'},
    273: {'negeri': 'PERAK', 'parlimen': 'BATU GAJAH', 'dun': 'JELAPANG'},
    274: {'negeri': 'PERAK', 'parlimen': 'BATU GAJAH', 'dun': 'MENGLEMBU'},
    275: {'negeri': 'PERAK', 'parlimen': 'BATU GAJAH', 'dun': 'TRONOH'},
    276: {
        'negeri': 'PERAK',
        'parlimen': 'KUALA KANGSAR',
        'dun': 'BUKIT CHANDAN',
    },
    277: {'negeri': 'PERAK', 'parlimen': 'KUALA KANGSAR', 'dun': 'MANONG'},
    278: {'negeri': 'PERAK', 'parlimen': 'BERUAS', 'dun': 'PENGKALAN BAHARU'},
    279: {'negeri': 'PERAK', 'parlimen': 'BERUAS', 'dun': 'PANTAI REMIS'},
    280: {'negeri': 'PERAK', 'parlimen': 'PARIT', 'dun': 'BELANJA'},
    281: {'negeri': 'PERAK', 'parlimen': 'PARIT', 'dun': 'BOTA'},
    282: {'negeri': 'PERAK', 'parlimen': 'KAMPAR', 'dun': 'MALIM NAWAR'},
    283: {'negeri': 'PERAK', 'parlimen': 'KAMPAR', 'dun': 'KERANJI'},
    284: {'negeri': 'PERAK', 'parlimen': 'KAMPAR', 'dun': 'TUALANG SEKAH'},
    285: {'negeri': 'PERAK', 'parlimen': 'GOPENG', 'dun': 'SUNGAI RAPAT'},
    286: {'negeri': 'PERAK', 'parlimen': 'GOPENG', 'dun': 'SIMPANG PULAI'},
    287: {'negeri': 'PERAK', 'parlimen': 'GOPENG', 'dun': 'TEJA'},
    288: {'negeri': 'PERAK', 'parlimen': 'TAPAH', 'dun': 'CHENDERIANG'},
    289: {'negeri': 'PERAK', 'parlimen': 'TAPAH', 'dun': 'AYER KUNING'},
    290: {'negeri': 'PERAK', 'parlimen': 'PASIR SALAK', 'dun': 'SUNGAI MANIK'},
    291: {'negeri': 'PERAK', 'parlimen': 'PASIR SALAK', 'dun': 'KAMPONG GAJAH'},
    292: {'negeri': 'PERAK', 'parlimen': 'LUMUT', 'dun': 'SITIAWAN'},
    293: {'negeri': 'PERAK', 'parlimen': 'LUMUT', 'dun': 'PASIR PANJANG'},
    294: {'negeri': 'PERAK', 'parlimen': 'LUMUT', 'dun': 'PANGKOR'},
    295: {'negeri': 'PERAK', 'parlimen': 'BAGAN DATOK', 'dun': 'RUNGKUP'},
    296: {
        'negeri': 'PERAK',
        'parlimen': 'BAGAN DATOK',
        'dun': 'HUTAN MELINTANG',
    },
    297: {'negeri': 'PERAK', 'parlimen': 'TELOK INTAN', 'dun': 'PASIR BEDAMAR'},
    298: {'negeri': 'PERAK', 'parlimen': 'TELOK INTAN', 'dun': 'CHANGKAT JONG'},
    299: {'negeri': 'PERAK', 'parlimen': 'TANJONG MALIM', 'dun': 'SUNGKAI'},
    300: {'negeri': 'PERAK', 'parlimen': 'TANJONG MALIM', 'dun': 'SLIM'},
    301: {'negeri': 'PERAK', 'parlimen': 'TANJONG MALIM', 'dun': 'BEHRANG'},
    302: {'negeri': 'PERLIS', 'parlimen': 'PADANG BESAR', 'dun': 'TITI TINGGI'},
    303: {'negeri': 'PERLIS', 'parlimen': 'PADANG BESAR', 'dun': 'BESERI'},
    304: {'negeri': 'PERLIS', 'parlimen': 'PADANG BESAR', 'dun': 'CHUPING'},
    305: {'negeri': 'PERLIS', 'parlimen': 'PADANG BESAR', 'dun': 'MATA AYER'},
    306: {'negeri': 'PERLIS', 'parlimen': 'PADANG BESAR', 'dun': 'SANTAN'},
    307: {'negeri': 'PERLIS', 'parlimen': 'KANGAR', 'dun': 'BINTONG'},
    308: {'negeri': 'PERLIS', 'parlimen': 'KANGAR', 'dun': 'SENA'},
    309: {'negeri': 'PERLIS', 'parlimen': 'KANGAR', 'dun': 'INDERA KAYANGAN'},
    310: {'negeri': 'PERLIS', 'parlimen': 'KANGAR', 'dun': 'KUALA PERLIS'},
    311: {'negeri': 'PERLIS', 'parlimen': 'KANGAR', 'dun': 'KAYANG'},
    312: {'negeri': 'PERLIS', 'parlimen': 'ARAU', 'dun': 'PAUH'},
    313: {'negeri': 'PERLIS', 'parlimen': 'ARAU', 'dun': 'TAMBUN TULANG'},
    314: {'negeri': 'PERLIS', 'parlimen': 'ARAU', 'dun': 'GUAR SANJI'},
    315: {'negeri': 'PERLIS', 'parlimen': 'ARAU', 'dun': 'SIMPANG EMPAT'},
    316: {'negeri': 'PERLIS', 'parlimen': 'ARAU', 'dun': 'SANGLANG'},
    317: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'KEPALA BATAS',
        'dun': 'PENAGA',
    },
    318: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'KEPALA BATAS',
        'dun': 'BERTAM',
    },
    319: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'KEPALA BATAS',
        'dun': 'PINANG TUNGGAL',
    },
    320: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'TASEK GELUGOR',
        'dun': 'PERMATANG BERANGAN',
    },
    321: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'TASEK GELUGOR',
        'dun': 'SUNGAI DUA',
    },
    322: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'TASEK GELUGOR',
        'dun': 'TELOK AYER TAWAR',
    },
    323: {'negeri': 'PULAU PINANG', 'parlimen': 'BAGAN', 'dun': 'SUNGAI PUYU'},
    324: {'negeri': 'PULAU PINANG', 'parlimen': 'BAGAN', 'dun': 'BAGAN JERMAL'},
    325: {'negeri': 'PULAU PINANG', 'parlimen': 'BAGAN', 'dun': 'BAGAN DALAM'},
    326: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'PERMATANG PAUH',
        'dun': 'SEBERANG JAYA',
    },
    327: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'PERMATANG PAUH',
        'dun': 'PERMATANG PASIR',
    },
    328: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'PERMATANG PAUH',
        'dun': 'PENANTI',
    },
    329: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BUKIT MERTAJAM',
        'dun': 'BERAPIT',
    },
    330: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BUKIT MERTAJAM',
        'dun': 'MACHANG BUBUK',
    },
    331: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BUKIT MERTAJAM',
        'dun': 'PADANG LALANG',
    },
    332: {'negeri': 'PULAU PINANG', 'parlimen': 'BATU KAWAN', 'dun': 'PERAI'},
    333: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BATU KAWAN',
        'dun': 'BUKIT TENGAH',
    },
    334: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BATU KAWAN',
        'dun': 'BUKIT TAMBUN',
    },
    335: {'negeri': 'PULAU PINANG', 'parlimen': 'NIBONG TEBAL', 'dun': 'JAWI'},
    336: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'NIBONG TEBAL',
        'dun': 'SUNGAI BAKAP',
    },
    337: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'NIBONG TEBAL',
        'dun': 'SUNGAI ACHEH',
    },
    338: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BUKIT BENDERA',
        'dun': 'TANJONG BUNGA',
    },
    339: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BUKIT BENDERA',
        'dun': 'AIR PUTIH',
    },
    340: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BUKIT BENDERA',
        'dun': 'KEBUN BUNGA',
    },
    341: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BUKIT BENDERA',
        'dun': 'PULAU TIKUS',
    },
    342: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'TANJONG',
        'dun': 'PADANG KOTA',
    },
    343: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'TANJONG',
        'dun': 'PENGKALAN KOTA',
    },
    344: {'negeri': 'PULAU PINANG', 'parlimen': 'TANJONG', 'dun': 'KOMTAR'},
    345: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'JELUTONG',
        'dun': 'DATOK KERAMAT',
    },
    346: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'JELUTONG',
        'dun': 'SUNGAI PINANG',
    },
    347: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'JELUTONG',
        'dun': 'BATU LANCANG',
    },
    348: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BUKIT GELUGOR',
        'dun': 'SERI DELIMA',
    },
    349: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BUKIT GELUGOR',
        'dun': 'AIR ITAM',
    },
    350: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BUKIT GELUGOR',
        'dun': 'PAYA TERUBONG',
    },
    351: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BAYAN BARU',
        'dun': 'BATU UBAN',
    },
    352: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BAYAN BARU',
        'dun': 'PANTAI JEREJAK',
    },
    353: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BAYAN BARU',
        'dun': 'BATU MAUNG',
    },
    354: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BALIK PULAU',
        'dun': 'BAYAN LEPAS',
    },
    355: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BALIK PULAU',
        'dun': 'PULAU BETONG',
    },
    356: {
        'negeri': 'PULAU PINANG',
        'parlimen': 'BALIK PULAU',
        'dun': 'TELOK BAHANG',
    },
    357: {'negeri': 'SABAH', 'parlimen': 'KUDAT', 'dun': 'BANGGI'},
    358: {'negeri': 'SABAH', 'parlimen': 'KUDAT', 'dun': 'TANJONG KAPOR'},
    359: {'negeri': 'SABAH', 'parlimen': 'KUDAT', 'dun': 'PITAS'},
    360: {'negeri': 'SABAH', 'parlimen': 'KOTA MARUDU', 'dun': 'MATUNGGONG'},
    361: {'negeri': 'SABAH', 'parlimen': 'KOTA MARUDU', 'dun': 'TANDEK'},
    362: {'negeri': 'SABAH', 'parlimen': 'KOTA BELUD', 'dun': 'TEMPASUK'},
    363: {'negeri': 'SABAH', 'parlimen': 'KOTA BELUD', 'dun': 'KADAMAIAN'},
    364: {'negeri': 'SABAH', 'parlimen': 'KOTA BELUD', 'dun': 'USUKAN'},
    365: {'negeri': 'SABAH', 'parlimen': 'TUARAN', 'dun': 'TAMPARULI'},
    366: {'negeri': 'SABAH', 'parlimen': 'TUARAN', 'dun': 'SULAMAN'},
    367: {'negeri': 'SABAH', 'parlimen': 'TUARAN', 'dun': 'KIULU'},
    368: {'negeri': 'SABAH', 'parlimen': 'SEPANGGAR', 'dun': 'KARAMBUNAI'},
    369: {'negeri': 'SABAH', 'parlimen': 'SEPANGGAR', 'dun': 'INANAM'},
    370: {'negeri': 'SABAH', 'parlimen': 'KOTA KINABALU', 'dun': 'LIKAS'},
    371: {'negeri': 'SABAH', 'parlimen': 'KOTA KINABALU', 'dun': 'API-API'},
    372: {'negeri': 'SABAH', 'parlimen': 'KOTA KINABALU', 'dun': 'LUYANG'},
    373: {'negeri': 'SABAH', 'parlimen': 'PUTATAN', 'dun': 'TANJONG ARU'},
    374: {'negeri': 'SABAH', 'parlimen': 'PUTATAN', 'dun': 'PETAGAS'},
    375: {'negeri': 'SABAH', 'parlimen': 'PENAMPANG', 'dun': 'KAPAYAN'},
    376: {'negeri': 'SABAH', 'parlimen': 'PENAMPANG', 'dun': 'MOYOG'},
    377: {'negeri': 'SABAH', 'parlimen': 'PAPAR', 'dun': 'KAWANG'},
    378: {'negeri': 'SABAH', 'parlimen': 'PAPAR', 'dun': 'PANTAI MANIS'},
    379: {'negeri': 'SABAH', 'parlimen': 'KIMANIS', 'dun': 'BONGAWAN'},
    380: {'negeri': 'SABAH', 'parlimen': 'KIMANIS', 'dun': 'MEMBAKUT'},
    381: {'negeri': 'SABAH', 'parlimen': 'BEAUFORT', 'dun': 'KLIAS'},
    382: {'negeri': 'SABAH', 'parlimen': 'BEAUFORT', 'dun': 'KUALA PENYU'},
    383: {'negeri': 'SABAH', 'parlimen': 'SIPITANG', 'dun': 'LUMADAN'},
    384: {'negeri': 'SABAH', 'parlimen': 'SIPITANG', 'dun': 'SINDUMIN'},
    385: {'negeri': 'SABAH', 'parlimen': 'RANAU', 'dun': 'KUNDASANG'},
    386: {'negeri': 'SABAH', 'parlimen': 'RANAU', 'dun': 'KARANAAN'},
    387: {'negeri': 'SABAH', 'parlimen': 'RANAU', 'dun': 'PAGINATAN'},
    388: {'negeri': 'SABAH', 'parlimen': 'KENINGAU', 'dun': 'TAMBUNAN'},
    389: {'negeri': 'SABAH', 'parlimen': 'KENINGAU', 'dun': 'BINGKOR'},
    390: {'negeri': 'SABAH', 'parlimen': 'KENINGAU', 'dun': 'LIAWAN'},
    391: {'negeri': 'SABAH', 'parlimen': 'TENOM', 'dun': 'MELALAP'},
    392: {'negeri': 'SABAH', 'parlimen': 'TENOM', 'dun': 'KEMABONG'},
    393: {'negeri': 'SABAH', 'parlimen': 'PENSIANGAN', 'dun': 'SOOK'},
    394: {'negeri': 'SABAH', 'parlimen': 'PENSIANGAN', 'dun': 'NABAWAN'},
    395: {'negeri': 'SABAH', 'parlimen': 'BELURAN', 'dun': 'SUGUT'},
    396: {'negeri': 'SABAH', 'parlimen': 'BELURAN', 'dun': 'LABUK'},
    397: {'negeri': 'SABAH', 'parlimen': 'LIBARAN', 'dun': 'GUM-GUM'},
    398: {'negeri': 'SABAH', 'parlimen': 'LIBARAN', 'dun': 'SUNGAI SIBUGA'},
    399: {'negeri': 'SABAH', 'parlimen': 'BATU SAPI', 'dun': 'SEKONG'},
    400: {'negeri': 'SABAH', 'parlimen': 'BATU SAPI', 'dun': 'KARAMUNTING'},
    401: {'negeri': 'SABAH', 'parlimen': 'SANDAKAN', 'dun': 'ELOPURA'},
    402: {'negeri': 'SABAH', 'parlimen': 'SANDAKAN', 'dun': 'TANJONG PAPAT'},
    403: {'negeri': 'SABAH', 'parlimen': 'KINABATANGAN', 'dun': 'KUAMUT'},
    404: {'negeri': 'SABAH', 'parlimen': 'KINABATANGAN', 'dun': 'SUKAU'},
    405: {'negeri': 'SABAH', 'parlimen': 'SILAM', 'dun': 'TUNGKU'},
    406: {'negeri': 'SABAH', 'parlimen': 'SILAM', 'dun': 'LAHAD DATU'},
    407: {'negeri': 'SABAH', 'parlimen': 'SILAM', 'dun': 'KUNAK'},
    408: {'negeri': 'SABAH', 'parlimen': 'SEMPORNA', 'dun': 'SULABAYAN'},
    409: {'negeri': 'SABAH', 'parlimen': 'SEMPORNA', 'dun': 'SENALLANG'},
    410: {'negeri': 'SABAH', 'parlimen': 'SEMPORNA', 'dun': 'BUGAYA'},
    411: {'negeri': 'SABAH', 'parlimen': 'TAWAU', 'dun': 'BALUNG'},
    412: {'negeri': 'SABAH', 'parlimen': 'TAWAU', 'dun': 'APAS'},
    413: {'negeri': 'SABAH', 'parlimen': 'TAWAU', 'dun': 'SRI TANJONG'},
    414: {'negeri': 'SABAH', 'parlimen': 'KALABAKAN', 'dun': 'MEROTAI'},
    415: {'negeri': 'SABAH', 'parlimen': 'KALABAKAN', 'dun': 'TANJONG BATU'},
    416: {'negeri': 'SABAH', 'parlimen': 'KALABAKAN', 'dun': 'SEBATIK'},
    417: {'negeri': 'SARAWAK', 'parlimen': 'MAS GADING', 'dun': 'OPAR'},
    418: {'negeri': 'SARAWAK', 'parlimen': 'MAS GADING', 'dun': 'TASIK BIRU'},
    419: {'negeri': 'SARAWAK', 'parlimen': 'SANTUBONG', 'dun': 'TANJONG DATU'},
    420: {'negeri': 'SARAWAK', 'parlimen': 'SANTUBONG', 'dun': 'PANTAI DAMAI'},
    421: {'negeri': 'SARAWAK', 'parlimen': 'SANTUBONG', 'dun': 'DEMAK LAUT'},
    422: {'negeri': 'SARAWAK', 'parlimen': 'PETRA JAYA', 'dun': 'TUPONG'},
    423: {'negeri': 'SARAWAK', 'parlimen': 'PETRA JAYA', 'dun': 'SAMARIANG'},
    424: {'negeri': 'SARAWAK', 'parlimen': 'PETRA JAYA', 'dun': 'SATOK'},
    425: {'negeri': 'SARAWAK', 'parlimen': 'BANDAR KUCHING', 'dun': 'PADUNGAN'},
    426: {'negeri': 'SARAWAK', 'parlimen': 'BANDAR KUCHING', 'dun': 'PENDING'},
    427: {
        'negeri': 'SARAWAK',
        'parlimen': 'BANDAR KUCHING',
        'dun': 'BATU LINTANG',
    },
    428: {'negeri': 'SARAWAK', 'parlimen': 'STAMPIN', 'dun': 'KOTA SENTOSA'},
    429: {'negeri': 'SARAWAK', 'parlimen': 'STAMPIN', 'dun': 'BATU KITANG'},
    430: {'negeri': 'SARAWAK', 'parlimen': 'STAMPIN', 'dun': 'BATU KAWAH'},
    431: {'negeri': 'SARAWAK', 'parlimen': 'KOTA SAMARAHAN', 'dun': 'ASAJAYA'},
    432: {
        'negeri': 'SARAWAK',
        'parlimen': 'KOTA SAMARAHAN',
        'dun': 'MUARA TUANG',
    },
    433: {'negeri': 'SARAWAK', 'parlimen': 'KOTA SAMARAHAN', 'dun': 'STAKAN'},
    434: {'negeri': 'SARAWAK', 'parlimen': 'MAMBONG', 'dun': 'SEREMBU'},
    435: {'negeri': 'SARAWAK', 'parlimen': 'MAMBONG', 'dun': 'MAMBONG'},
    436: {'negeri': 'SARAWAK', 'parlimen': 'MAMBONG', 'dun': 'TARAT'},
    437: {'negeri': 'SARAWAK', 'parlimen': 'SERIAN', 'dun': 'TEBEDU'},
    438: {'negeri': 'SARAWAK', 'parlimen': 'SERIAN', 'dun': 'KEDUP'},
    439: {'negeri': 'SARAWAK', 'parlimen': 'SERIAN', 'dun': 'BUKIT SEMUJA'},
    440: {
        'negeri': 'SARAWAK',
        'parlimen': 'BATANG SADONG',
        'dun': 'SADONG JAYA',
    },
    441: {'negeri': 'SARAWAK', 'parlimen': 'BATANG SADONG', 'dun': 'SIMUNJAN'},
    442: {'negeri': 'SARAWAK', 'parlimen': 'BATANG SADONG', 'dun': 'GEDONG'},
    443: {'negeri': 'SARAWAK', 'parlimen': 'BATANG LUPAR', 'dun': 'SEBUYAU'},
    444: {'negeri': 'SARAWAK', 'parlimen': 'BATANG LUPAR', 'dun': 'LINGGA'},
    445: {
        'negeri': 'SARAWAK',
        'parlimen': 'BATANG LUPAR',
        'dun': 'BETING MARO',
    },
    446: {'negeri': 'SARAWAK', 'parlimen': 'SRI AMAN', 'dun': 'BALAI RINGIN'},
    447: {'negeri': 'SARAWAK', 'parlimen': 'SRI AMAN', 'dun': 'BUKIT BEGUNAN'},
    448: {'negeri': 'SARAWAK', 'parlimen': 'SRI AMAN', 'dun': 'SIMANGGANG'},
    449: {'negeri': 'SARAWAK', 'parlimen': 'LUBOK ANTU', 'dun': 'ENGKILILI'},
    450: {'negeri': 'SARAWAK', 'parlimen': 'LUBOK ANTU', 'dun': 'BATANG AI'},
    451: {'negeri': 'SARAWAK', 'parlimen': 'BETONG', 'dun': 'SARIBAS'},
    452: {'negeri': 'SARAWAK', 'parlimen': 'BETONG', 'dun': 'LAYAR'},
    453: {'negeri': 'SARAWAK', 'parlimen': 'BETONG', 'dun': 'BUKIT SABAN'},
    454: {'negeri': 'SARAWAK', 'parlimen': 'SARATOK', 'dun': 'KALAKA'},
    455: {'negeri': 'SARAWAK', 'parlimen': 'SARATOK', 'dun': 'KRIAN'},
    456: {'negeri': 'SARAWAK', 'parlimen': 'SARATOK', 'dun': 'KABONG'},
    457: {
        'negeri': 'SARAWAK',
        'parlimen': 'TANJONG MANIS',
        'dun': 'KUALA RAJANG',
    },
    458: {'negeri': 'SARAWAK', 'parlimen': 'TANJONG MANIS', 'dun': 'SEMOP'},
    459: {'negeri': 'SARAWAK', 'parlimen': 'IGAN', 'dun': 'DARO'},
    460: {'negeri': 'SARAWAK', 'parlimen': 'IGAN', 'dun': 'JEMORENG'},
    461: {'negeri': 'SARAWAK', 'parlimen': 'SARIKEI', 'dun': 'REPOK'},
    462: {'negeri': 'SARAWAK', 'parlimen': 'SARIKEI', 'dun': 'MERADONG'},
    463: {'negeri': 'SARAWAK', 'parlimen': 'JULAU', 'dun': 'PAKAN'},
    464: {'negeri': 'SARAWAK', 'parlimen': 'JULAU', 'dun': 'MELUAN'},
    465: {'negeri': 'SARAWAK', 'parlimen': 'KANOWIT', 'dun': 'NGEMAH'},
    466: {'negeri': 'SARAWAK', 'parlimen': 'KANOWIT', 'dun': 'MACHAN'},
    467: {'negeri': 'SARAWAK', 'parlimen': 'LANANG', 'dun': 'BUKIT ASSEK'},
    468: {'negeri': 'SARAWAK', 'parlimen': 'LANANG', 'dun': 'DUDONG'},
    469: {'negeri': 'SARAWAK', 'parlimen': 'SIBU', 'dun': 'BAWANG ASSAN'},
    470: {'negeri': 'SARAWAK', 'parlimen': 'SIBU', 'dun': 'PELAWAN'},
    471: {'negeri': 'SARAWAK', 'parlimen': 'SIBU', 'dun': 'NANGKA'},
    472: {'negeri': 'SARAWAK', 'parlimen': 'MUKAH', 'dun': 'DALAT'},
    473: {'negeri': 'SARAWAK', 'parlimen': 'MUKAH', 'dun': 'TELLIAN'},
    474: {'negeri': 'SARAWAK', 'parlimen': 'MUKAH', 'dun': 'BALINGIAN'},
    475: {'negeri': 'SARAWAK', 'parlimen': 'SELANGAU', 'dun': 'TAMIN'},
    476: {'negeri': 'SARAWAK', 'parlimen': 'SELANGAU', 'dun': 'KAKUS'},
    477: {'negeri': 'SARAWAK', 'parlimen': 'KAPIT', 'dun': 'PELAGUS'},
    478: {'negeri': 'SARAWAK', 'parlimen': 'KAPIT', 'dun': 'KATIBAS'},
    479: {'negeri': 'SARAWAK', 'parlimen': 'KAPIT', 'dun': 'BUKIT GORAM'},
    480: {'negeri': 'SARAWAK', 'parlimen': 'HULU RAJANG', 'dun': 'BALEH'},
    481: {'negeri': 'SARAWAK', 'parlimen': 'HULU RAJANG', 'dun': 'BELAGA'},
    482: {'negeri': 'SARAWAK', 'parlimen': 'HULU RAJANG', 'dun': 'MURUM'},
    483: {'negeri': 'SARAWAK', 'parlimen': 'BINTULU', 'dun': 'JEPAK'},
    484: {'negeri': 'SARAWAK', 'parlimen': 'BINTULU', 'dun': 'TANJONG BATU'},
    485: {'negeri': 'SARAWAK', 'parlimen': 'BINTULU', 'dun': 'KEMENA'},
    486: {'negeri': 'SARAWAK', 'parlimen': 'BINTULU', 'dun': 'SAMALAJU'},
    487: {'negeri': 'SARAWAK', 'parlimen': 'SIBUTI', 'dun': 'BEKENU'},
    488: {'negeri': 'SARAWAK', 'parlimen': 'SIBUTI', 'dun': 'LAMBIR'},
    489: {'negeri': 'SARAWAK', 'parlimen': 'MIRI', 'dun': 'PIASAU'},
    490: {'negeri': 'SARAWAK', 'parlimen': 'MIRI', 'dun': 'PUJUT'},
    491: {'negeri': 'SARAWAK', 'parlimen': 'MIRI', 'dun': 'SENADIN'},
    492: {'negeri': 'SARAWAK', 'parlimen': 'BARAM', 'dun': 'MARUDI'},
    493: {'negeri': 'SARAWAK', 'parlimen': 'BARAM', 'dun': 'TELANG USAN'},
    494: {'negeri': 'SARAWAK', 'parlimen': 'BARAM', 'dun': 'MULU'},
    495: {'negeri': 'SARAWAK', 'parlimen': 'LIMBANG', 'dun': 'BUKIT KOTA'},
    496: {'negeri': 'SARAWAK', 'parlimen': 'LIMBANG', 'dun': 'BATU DANAU'},
    497: {'negeri': 'SARAWAK', 'parlimen': 'LAWAS', 'dun': 'BA`KELALAN'},
    498: {'negeri': 'SARAWAK', 'parlimen': 'LAWAS', 'dun': 'BUKIT SARI'},
    499: {
        'negeri': 'SELANGOR',
        'parlimen': 'SABAK BERNAM',
        'dun': 'SUNGAI AIR TAWAR',
    },
    500: {'negeri': 'SELANGOR', 'parlimen': 'SABAK BERNAM', 'dun': 'SABAK'},
    501: {
        'negeri': 'SELANGOR',
        'parlimen': 'SUNGAI BESAR',
        'dun': 'SUNGAI PANJANG',
    },
    502: {'negeri': 'SELANGOR', 'parlimen': 'SUNGAI BESAR', 'dun': 'SEKINCHAN'},
    503: {
        'negeri': 'SELANGOR',
        'parlimen': 'HULU SELANGOR',
        'dun': 'HULU BERNAM',
    },
    504: {
        'negeri': 'SELANGOR',
        'parlimen': 'HULU SELANGOR',
        'dun': 'KUALA KUBU BAHARU',
    },
    505: {
        'negeri': 'SELANGOR',
        'parlimen': 'HULU SELANGOR',
        'dun': 'BATANG KALI',
    },
    506: {
        'negeri': 'SELANGOR',
        'parlimen': 'TANJONG KARANG',
        'dun': 'SUNGAI BURONG',
    },
    507: {
        'negeri': 'SELANGOR',
        'parlimen': 'TANJONG KARANG',
        'dun': 'PERMATANG',
    },
    508: {
        'negeri': 'SELANGOR',
        'parlimen': 'KUALA SELANGOR',
        'dun': 'BUKIT MELAWATI',
    },
    509: {'negeri': 'SELANGOR', 'parlimen': 'KUALA SELANGOR', 'dun': 'IJOK'},
    510: {'negeri': 'SELANGOR', 'parlimen': 'KUALA SELANGOR', 'dun': 'JERAM'},
    511: {'negeri': 'SELANGOR', 'parlimen': 'SELAYANG', 'dun': 'KUANG'},
    512: {'negeri': 'SELANGOR', 'parlimen': 'SELAYANG', 'dun': 'RAWANG'},
    513: {'negeri': 'SELANGOR', 'parlimen': 'SELAYANG', 'dun': 'TAMAN TEMPLER'},
    514: {'negeri': 'SELANGOR', 'parlimen': 'GOMBAK', 'dun': 'BATU CAVES'},
    515: {'negeri': 'SELANGOR', 'parlimen': 'GOMBAK', 'dun': 'GOMBAK SETIA'},
    516: {'negeri': 'SELANGOR', 'parlimen': 'GOMBAK', 'dun': 'HULU KELANG'},
    517: {
        'negeri': 'SELANGOR',
        'parlimen': 'AMPANG',
        'dun': 'BUKIT ANTARABANGSA',
    },
    518: {'negeri': 'SELANGOR', 'parlimen': 'AMPANG', 'dun': 'LEMBAH JAYA'},
    519: {'negeri': 'SELANGOR', 'parlimen': 'PANDAN', 'dun': 'CHEMPAKA'},
    520: {'negeri': 'SELANGOR', 'parlimen': 'PANDAN', 'dun': 'TERATAI'},
    521: {'negeri': 'SELANGOR', 'parlimen': 'HULU LANGAT', 'dun': 'DUSUN TUA'},
    522: {'negeri': 'SELANGOR', 'parlimen': 'HULU LANGAT', 'dun': 'SEMENYIH'},
    523: {'negeri': 'SELANGOR', 'parlimen': 'HULU LANGAT', 'dun': 'KAJANG'},
    524: {'negeri': 'SELANGOR', 'parlimen': 'SERDANG', 'dun': 'BANGI'},
    525: {'negeri': 'SELANGOR', 'parlimen': 'SERDANG', 'dun': 'BALAKONG'},
    526: {'negeri': 'SELANGOR', 'parlimen': 'SERDANG', 'dun': 'SERI KEMBANGAN'},
    527: {'negeri': 'SELANGOR', 'parlimen': 'PUCHONG', 'dun': 'SERI SERDANG'},
    528: {'negeri': 'SELANGOR', 'parlimen': 'PUCHONG', 'dun': 'KINRARA'},
    529: {
        'negeri': 'SELANGOR',
        'parlimen': 'KELANA JAYA',
        'dun': 'SUBANG JAYA',
    },
    530: {'negeri': 'SELANGOR', 'parlimen': 'KELANA JAYA', 'dun': 'SERI SETIA'},
    531: {
        'negeri': 'SELANGOR',
        'parlimen': 'PETALING JAYA SELATAN',
        'dun': 'TAMAN MEDAN',
    },
    532: {
        'negeri': 'SELANGOR',
        'parlimen': 'PETALING JAYA SELATAN',
        'dun': 'BUKIT GASING',
    },
    533: {
        'negeri': 'SELANGOR',
        'parlimen': 'PETALING JAYA UTARA',
        'dun': 'KAMPUNG TUNKU',
    },
    534: {
        'negeri': 'SELANGOR',
        'parlimen': 'PETALING JAYA UTARA',
        'dun': 'DAMANSARA UTAMA',
    },
    535: {'negeri': 'SELANGOR', 'parlimen': 'SUBANG', 'dun': 'BUKIT LANJAN'},
    536: {'negeri': 'SELANGOR', 'parlimen': 'SUBANG', 'dun': 'PAYA JARAS'},
    537: {'negeri': 'SELANGOR', 'parlimen': 'SUBANG', 'dun': 'KOTA DAMANSARA'},
    538: {
        'negeri': 'SELANGOR',
        'parlimen': 'SHAH ALAM',
        'dun': 'KOTA ANGGERIK',
    },
    539: {'negeri': 'SELANGOR', 'parlimen': 'SHAH ALAM', 'dun': 'BATU TIGA'},
    540: {'negeri': 'SELANGOR', 'parlimen': 'KAPAR', 'dun': 'MERU'},
    541: {'negeri': 'SELANGOR', 'parlimen': 'KAPAR', 'dun': 'SEMENTA'},
    542: {'negeri': 'SELANGOR', 'parlimen': 'KAPAR', 'dun': 'SUNGAI PINANG'},
    543: {'negeri': 'SELANGOR', 'parlimen': 'KAPAR', 'dun': 'SELAT KLANG'},
    544: {'negeri': 'SELANGOR', 'parlimen': 'KLANG', 'dun': 'PELABUHAN KLANG'},
    545: {'negeri': 'SELANGOR', 'parlimen': 'KLANG', 'dun': 'PANDAMARAN'},
    546: {'negeri': 'SELANGOR', 'parlimen': 'KLANG', 'dun': 'KOTA ALAM SHAH'},
    547: {'negeri': 'SELANGOR', 'parlimen': 'KOTA RAJA', 'dun': 'SERI ANDALAS'},
    548: {'negeri': 'SELANGOR', 'parlimen': 'KOTA RAJA', 'dun': 'SRI MUDA'},
    549: {
        'negeri': 'SELANGOR',
        'parlimen': 'KUALA LANGAT',
        'dun': 'SIJANGKANG',
    },
    550: {
        'negeri': 'SELANGOR',
        'parlimen': 'KUALA LANGAT',
        'dun': 'TELUK DATUK',
    },
    551: {'negeri': 'SELANGOR', 'parlimen': 'KUALA LANGAT', 'dun': 'MORIB'},
    552: {'negeri': 'SELANGOR', 'parlimen': 'SEPANG', 'dun': 'TANJONG SEPAT'},
    553: {'negeri': 'SELANGOR', 'parlimen': 'SEPANG', 'dun': 'DENGKIL'},
    554: {'negeri': 'SELANGOR', 'parlimen': 'SEPANG', 'dun': 'SUNGAI PELEK'},
    555: {'negeri': 'TERENGGANU', 'parlimen': 'BESUT', 'dun': 'KUALA BESUT'},
    556: {'negeri': 'TERENGGANU', 'parlimen': 'BESUT', 'dun': 'KOTA PUTERA'},
    557: {'negeri': 'TERENGGANU', 'parlimen': 'BESUT', 'dun': 'JERTIH'},
    558: {'negeri': 'TERENGGANU', 'parlimen': 'BESUT', 'dun': 'HULU BESUT'},
    559: {'negeri': 'TERENGGANU', 'parlimen': 'SETIU', 'dun': 'JABI'},
    560: {'negeri': 'TERENGGANU', 'parlimen': 'SETIU', 'dun': 'PERMAISURI'},
    561: {'negeri': 'TERENGGANU', 'parlimen': 'SETIU', 'dun': 'LANGKAP'},
    562: {'negeri': 'TERENGGANU', 'parlimen': 'SETIU', 'dun': 'BATU RAKIT'},
    563: {'negeri': 'TERENGGANU', 'parlimen': 'KUALA NERUS', 'dun': 'TEPUH'},
    564: {
        'negeri': 'TERENGGANU',
        'parlimen': 'KUALA NERUS',
        'dun': 'TELUK PASU',
    },
    565: {
        'negeri': 'TERENGGANU',
        'parlimen': 'KUALA NERUS',
        'dun': 'SEBERANG TAKIR',
    },
    566: {
        'negeri': 'TERENGGANU',
        'parlimen': 'KUALA NERUS',
        'dun': 'BUKIT TUNGGAL',
    },
    567: {
        'negeri': 'TERENGGANU',
        'parlimen': 'KUALA TERENGGANU',
        'dun': 'WAKAF MEMPELAM',
    },
    568: {
        'negeri': 'TERENGGANU',
        'parlimen': 'KUALA TERENGGANU',
        'dun': 'BANDAR',
    },
    569: {
        'negeri': 'TERENGGANU',
        'parlimen': 'KUALA TERENGGANU',
        'dun': 'LADANG',
    },
    570: {
        'negeri': 'TERENGGANU',
        'parlimen': 'KUALA TERENGGANU',
        'dun': 'BATU BURUK',
    },
    571: {'negeri': 'TERENGGANU', 'parlimen': 'MARANG', 'dun': 'ALUR LIMBAT'},
    572: {'negeri': 'TERENGGANU', 'parlimen': 'MARANG', 'dun': 'BUKIT PAYUNG'},
    573: {'negeri': 'TERENGGANU', 'parlimen': 'MARANG', 'dun': 'RU RENDANG'},
    574: {
        'negeri': 'TERENGGANU',
        'parlimen': 'MARANG',
        'dun': 'PENGKALAN BERANGAN',
    },
    575: {
        'negeri': 'TERENGGANU',
        'parlimen': 'HULU TERENGGANU',
        'dun': 'TELEMUNG',
    },
    576: {
        'negeri': 'TERENGGANU',
        'parlimen': 'HULU TERENGGANU',
        'dun': 'MANIR',
    },
    577: {
        'negeri': 'TERENGGANU',
        'parlimen': 'HULU TERENGGANU',
        'dun': 'KUALA BERANG',
    },
    578: {'negeri': 'TERENGGANU', 'parlimen': 'HULU TERENGGANU', 'dun': 'AJIL'},
    579: {'negeri': 'TERENGGANU', 'parlimen': 'DUNGUN', 'dun': 'BUKIT BESI'},
    580: {'negeri': 'TERENGGANU', 'parlimen': 'DUNGUN', 'dun': 'RANTAU ABANG'},
    581: {'negeri': 'TERENGGANU', 'parlimen': 'DUNGUN', 'dun': 'SURA'},
    582: {'negeri': 'TERENGGANU', 'parlimen': 'DUNGUN', 'dun': 'PAKA'},
    583: {'negeri': 'TERENGGANU', 'parlimen': 'KEMAMAN', 'dun': 'KEMASIK'},
    584: {'negeri': 'TERENGGANU', 'parlimen': 'KEMAMAN', 'dun': 'KIJAL'},
    585: {'negeri': 'TERENGGANU', 'parlimen': 'KEMAMAN', 'dun': 'CUKAI'},
    586: {'negeri': 'TERENGGANU', 'parlimen': 'KEMAMAN', 'dun': 'AIR PUTIH'},
    587: {'negeri': 'WILAYAH PERSEKUTUAN', 'parlimen': 'KEPONG', 'dun': 'NAN'},
    588: {
        'negeri': 'WILAYAH PERSEKUTUAN',
        'parlimen': 'WANGSA MAJU',
        'dun': 'NAN',
    },
    589: {
        'negeri': 'WILAYAH PERSEKUTUAN',
        'parlimen': 'SEGAMBUT',
        'dun': 'NAN',
    },
    590: {
        'negeri': 'WILAYAH PERSEKUTUAN',
        'parlimen': 'SETIAWANGSA',
        'dun': 'NAN',
    },
    591: {
        'negeri': 'WILAYAH PERSEKUTUAN',
        'parlimen': 'TITIWANGSA',
        'dun': 'NAN',
    },
    592: {
        'negeri': 'WILAYAH PERSEKUTUAN',
        'parlimen': 'BUKIT BINTANG',
        'dun': 'NAN',
    },
    593: {
        'negeri': 'WILAYAH PERSEKUTUAN',
        'parlimen': 'LEMBAH PANTAI',
        'dun': 'NAN',
    },
    594: {'negeri': 'WILAYAH PERSEKUTUAN', 'parlimen': 'SEPUTEH', 'dun': 'NAN'},
    595: {'negeri': 'WILAYAH PERSEKUTUAN', 'parlimen': 'CHERAS', 'dun': 'NAN'},
    596: {
        'negeri': 'WILAYAH PERSEKUTUAN',
        'parlimen': 'BANDAR TUN RAZAK',
        'dun': 'NAN',
    },
    597: {
        'negeri': 'WILAYAH PERSEKUTUAN',
        'parlimen': 'PUTRAJAYA',
        'dun': 'NAN',
    },
    598: {'negeri': 'WILAYAH PERSEKUTUAN', 'parlimen': 'LABUAN', 'dun': 'NAN'},
}



import malaya

def test_pretrained_bayes_sentiment():
    positive_text = 'Kerajaan negeri Kelantan mempersoalkan motif kenyataan Menteri Kewangan Lim Guan Eng yang hanya menyebut Kelantan penerima terbesar bantuan kewangan dari Kerajaan Persekutuan. Sedangkan menurut Timbalan Menteri Besarnya, Datuk Mohd Amar Nik Abdullah, negeri lain yang lebih maju dari Kelantan turut mendapat pembiayaan dan pinjaman.'
    news_sentiment = malaya.pretrained_bayes_sentiment()
    assert len(news_sentiment.predict(positive_text)) > 0

def test_pretrained_xgb_sentiment():
    positive_text = 'Kerajaan negeri Kelantan mempersoalkan motif kenyataan Menteri Kewangan Lim Guan Eng yang hanya menyebut Kelantan penerima terbesar bantuan kewangan dari Kerajaan Persekutuan. Sedangkan menurut Timbalan Menteri Besarnya, Datuk Mohd Amar Nik Abdullah, negeri lain yang lebih maju dari Kelantan turut mendapat pembiayaan dan pinjaman.'
    news_sentiment = malaya.pretrained_xgb_sentiment()
    assert len(news_sentiment.predict(positive_text)) > 0

def test_pretrained_bayes_sentiment_batch():
    positive_text = 'Kerajaan negeri Kelantan mempersoalkan motif kenyataan Menteri Kewangan Lim Guan Eng yang hanya menyebut Kelantan penerima terbesar bantuan kewangan dari Kerajaan Persekutuan. Sedangkan menurut Timbalan Menteri Besarnya, Datuk Mohd Amar Nik Abdullah, negeri lain yang lebih maju dari Kelantan turut mendapat pembiayaan dan pinjaman.'
    news_sentiment = malaya.pretrained_bayes_sentiment()
    assert len(news_sentiment.predict_batch([positive_text,positive_text])) > 0

def test_bahdanau_sentiment():
    malaya.get_available_sentiment_models()
    negative_text = 'kerajaan sebenarnya sangat bencikan rakyatnya, minyak naik dan segalanya'
    news_sentiment = malaya.deep_sentiment('bahdanau')
    assert len(news_sentiment.predict(negative_text)['attention']) > 1

def test_attention_sentiment():
    malaya.get_available_sentiment_models()
    negative_text = 'kerajaan sebenarnya sangat bencikan rakyatnya, minyak naik dan segalanya'
    news_sentiment = malaya.deep_sentiment('hierarchical')
    assert len(news_sentiment.predict(negative_text)['attention']) > 1

def test_luong_sentiment():
    malaya.get_available_sentiment_models()
    negative_text = 'kerajaan sebenarnya sangat bencikan rakyatnya, minyak naik dan segalanya'
    news_sentiment = malaya.deep_sentiment('luong')
    assert len(news_sentiment.predict(negative_text)['attention']) > 1

def test_normal_sentiment():
    malaya.get_available_sentiment_models()
    negative_text = 'kerajaan sebenarnya sangat bencikan rakyatnya, minyak naik dan segalanya'
    news_sentiment = malaya.deep_sentiment('bidirectional')
    assert news_sentiment.predict(negative_text)['negative'] > 0

def test_fasttext_sentiment():
    malaya.get_available_sentiment_models()
    negative_text = 'kerajaan sebenarnya sangat bencikan rakyatnya, minyak naik dan segalanya'
    news_sentiment = malaya.deep_sentiment('fast-text')
    assert news_sentiment.predict(negative_text)['negative'] > 0

def test_normal_sentiment_batch():
    malaya.get_available_sentiment_models()
    negative_text = 'kerajaan sebenarnya sangat bencikan rakyatnya, minyak naik dan segalanya'
    news_sentiment = malaya.deep_sentiment('bidirectional')
    assert len(news_sentiment.predict_batch([negative_text,negative_text])) > 0

def test_fasttext_sentiment_batch():
    malaya.get_available_sentiment_models()
    negative_text = 'kerajaan sebenarnya sangat bencikan rakyatnya, minyak naik dan segalanya'
    news_sentiment = malaya.deep_sentiment('fast-text')
    assert len(news_sentiment.predict_batch([negative_text,negative_text])) > 0

def test_bayes_sentiment():
    import pandas as pd
    df = pd.read_csv('tests/02032018.csv',sep=';')
    df = df.iloc[3:,1:]
    df.columns = ['text','label']
    dataset = [[df.iloc[i,0],df.iloc[i,1]] for i in range(df.shape[0])]
    bayes=malaya.bayes_sentiment(dataset)
    assert len(bayes.predict(dataset[0][0])) > 0

def test_bayes_sentiment_bow_nosplit():
    import pandas as pd
    df = pd.read_csv('tests/02032018.csv',sep=';')
    df = df.iloc[3:,1:]
    df.columns = ['text','label']
    dataset = [[df.iloc[i,0],df.iloc[i,1]] for i in range(df.shape[0])]
    bayes=malaya.bayes_sentiment(dataset, vector = 'bow', split_size = None)
    assert len(bayes.predict(dataset[0][0])) > 0

def test_bayes_sentiment_location():
    bayes = malaya.bayes_sentiment('tests/local')
    assert len(bayes.predict('saya suka kerajaan dan anwar ibrahim')) > 0



import itertools
import random
from .texts._text_functions import simple_textcleaning

_accepted_pos = [
    'ADJ',
    'ADP',
    'ADV',
    'ADX',
    'CCONJ',
    'DET',
    'NOUN',
    'NUM',
    'PART',
    'PRON',
    'PROPN',
    'SCONJ',
    'SYM',
    'VERB',
    'X',
]
_accepted_entities = [
    'OTHER',
    'law',
    'location',
    'organization',
    'person',
    'quantity',
    'time',
    'event',
]


def _pad_sequence(
    sequence,
    n,
    pad_left = False,
    pad_right = False,
    left_pad_symbol = None,
    right_pad_symbol = None,
):
    sequence = iter(sequence)
    if pad_left:
        sequence = itertools.chain((left_pad_symbol,) * (n - 1), sequence)
    if pad_right:
        sequence = itertools.chain(sequence, (right_pad_symbol,) * (n - 1))
    return sequence


def ngrams(
    sequence,
    n,
    pad_left = False,
    pad_right = False,
    left_pad_symbol = None,
    right_pad_symbol = None,
):
    """
    generate ngrams

    Parameters
    ----------
    sequence : list of str
        list of tokenize words
    n : int
        ngram size

    Returns
    -------
    ngram: list
    """
    sequence = _pad_sequence(
        sequence, n, pad_left, pad_right, left_pad_symbol, right_pad_symbol
    )

    history = []
    while n > 1:
        try:
            next_item = next(sequence)
        except StopIteration:
            return
        history.append(next_item)
        n -= 1
    for item in sequence:
        history.append(item)
        yield tuple(history)
        del history[0]


def pos_entities_ngram(
    result_pos,
    result_entities,
    ngram = (1, 3),
    accept_pos = ['NOUN', 'PROPN', 'VERB'],
    accept_entities = ['law', 'location', 'organization', 'person', 'time'],
):
    """
    generate ngrams

    Parameters
    ----------
    result_pos : list of tuple
        result from POS recognition
    result_entities : list of tuple
        result of Entities recognition
    ngram : tuple
        ngram sizes
    accept_pos : list of str
        accepted POS elements
    accept_entities : list of str
        accept entities elements

    Returns
    -------
    result: list
    """
    if not isinstance(result_pos, list):
        raise ValueError('result_pos must be a list')
    if not isinstance(result_pos[0], tuple):
        raise ValueError('result_pos must be a list of tuple')
    if not isinstance(ngram, tuple):
        raise ValueError('ngram must be a tuple')
    if not len(ngram) == 2:
        raise ValueError('ngram size must equal to 2')
    if not isinstance(accept_pos, list):
        raise ValueError('accept_pos must be a list')
    if not isinstance(accept_entities, list):
        raise ValueError('accept_entites must be a list')
    if not all([i in _accepted_pos for i in accept_pos]):
        raise ValueError(
            'accept_pos must be a subset or equal of supported POS, please run malaya.describe_pos() to get supported POS'
        )
    if not all([i in _accepted_entities for i in accept_entities]):
        raise ValueError(
            'accept_entites must be a subset or equal of supported entities, please run malaya.describe_entities() to get supported entities'
        )

    words = []
    sentences = []
    for no in range(len(result_pos)):
        if (
            result_pos[no][1] in accept_pos
            or result_entities[no][1] in accept_entities
        ):
            words.append(result_pos[no][0])
    for gram in range(ngram[0], ngram[1] + 1, 1):
        gram_words = list(ngrams(words, gram))
        for sentence in gram_words:
            sentences.append(' '.join(sentence))
    return list(set(sentences))


def sentence_ngram(sentence, ngram = (1, 3)):
    """
    generate ngram for a text

    Parameters
    ----------
    sentence: str
    ngram : tuple
        ngram sizes

    Returns
    -------
    result: list
    """
    if not isinstance(sentence, str):
        raise ValueError('sentence must be a string')
    if not isinstance(ngram, tuple):
        raise ValueError('ngram must be a tuple')
    if not len(ngram) == 2:
        raise ValueError('ngram size must equal to 2')

    words = sentence.split()
    sentences = []
    for gram in range(ngram[0], ngram[1] + 1, 1):
        gram_words = list(ngrams(words, gram))
        for sentence in gram_words:
            sentences.append(' '.join(sentence))
    return list(set(sentences))


def w2v_augmentation(
    string,
    w2v,
    threshold = 0.5,
    soft = False,
    random_select = True,
    augment_counts = 1,
    top_n = 5,
    cleaning_function = simple_textcleaning,
):
    """
    generate ngram for a text

    Parameters
    ----------
    string: str
    w2v: object
        word2vec interface object
    threshold: float, optional (default=0.5)
        random selection for a word
    soft: bool, optional (default=False)
        if True, a word not in the dictionary will be replaced with nearest fuzzywuzzy ratio.
        if False, it will throw an exception if a word not in the dictionary
    random_select: bool, (default=True)
        if True, a word randomly selected in the pool
        if False, based on the index
    augment_counts: int, (default=1)
        augmentation count for a string
    top_n: int, (default=5)
        number of nearest neighbors returned
    cleaning_function: function, (default=simple_textcleaning)


    Returns
    -------
    result: list
    """
    if not isinstance(string, str):
        raise ValueError('string must be a string')
    if not isinstance(threshold, float):
        raise ValueError('threshold must be a float')
    if not (threshold > 0 and threshold < 1):
        raise ValueError('threshold must be bigger than 0 and less than 1')
    if not isinstance(soft, bool):
        raise ValueError('soft must be a boolean')
    if not hasattr(w2v, 'batch_n_closest'):
        raise ValueError('word2vec must has `batch_n_closest` method')
    if not isinstance(random_select, bool):
        raise ValueError('random_select must be a boolean')
    if not isinstance(top_n, int):
        raise ValueError('top_n must be an integer')
    if not isinstance(augment_counts, int):
        raise ValueError('augment_counts must be an integer')
    if not random_select:
        if augment_counts > top_n:
            raise ValueError(
                'if random_select is False, augment_counts need to be less than or equal to top_n'
            )
    if cleaning_function:
        string = cleaning_function(string)
    string = string.split()
    selected = []
    while not len(selected):
        selected = [
            (no, w)
            for no, w in enumerate(string)
            if random.random() > threshold
        ]
    indices, words = [i[0] for i in selected], [i[1] for i in selected]
    results = w2v.batch_n_closest(words, num_closest = top_n, soft = soft)
    augmented = []
    for i in range(augment_counts):
        string_ = string[:]
        for no in range(len(results)):
            if random_select:
                index = random.randint(0, len(results[no]) - 1)
            else:
                index = i
            string_[indices[no]] = results[no][index]
        augmented.append(' '.join(string_))
    return augmented



import xgboost as xgb
import numpy as np
from collections import Counter
from scipy.sparse import hstack
from ..texts._text_functions import (
    simple_textcleaning,
    classification_textcleaning,
    entities_textcleaning,
    language_detection_textcleaning,
)
from ..texts.vectorizer import features_crf, features_crf_dependency


def transitions(trans_features):
    for (label_from, label_to), weight in trans_features:
        print('%-6s -> %-7s %0.6f' % (label_from, label_to, weight))


def state_features(state_features):
    for (attr, label), weight in state_features:
        print('%0.6f %-8s %s' % (weight, label, attr))


class CRF:
    def __init__(self, model, is_lower = True):
        self._model = model
        self._is_lower = is_lower

    def predict(self, string):
        """
        Tag a string

        Parameters
        ----------
        string : str

        Returns
        -------
        string: tagged string
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        string = string.lower() if self._is_lower else string
        string = entities_textcleaning(string)
        batch_x = [features_crf(string, index) for index in range(len(string))]
        return [
            (string[no], tag)
            for no, tag in enumerate(self._model.predict_single(batch_x))
        ]

    def print_transitions(self, top_k = 10):
        """
        Print important top-k transitions

        Parameters
        ----------
        top_k : int
        """
        if not isinstance(top_k, int):
            raise ValueError('input must be an integer')
        print('Top-%d likely transitions:' % (top_k))
        transitions(
            Counter(self._model.transition_features_).most_common(top_k)
        )

        print('\nTop-%d unlikely transitions:' % (top_k))
        transitions(
            Counter(self._model.transition_features_).most_common()[-top_k:]
        )

    def print_features(self, top_k = 10):
        """
        Print important top-k features

        Parameters
        ----------
        top_k : int
        """
        if not isinstance(top_k, int):
            raise ValueError('input must be an integer')
        print('Top-%d positive:' % (top_k))
        state_features(Counter(self._model.state_features_).most_common(top_k))

        print('\nTop-%d negative:' % (top_k))
        state_features(
            Counter(self._model.state_features_).most_common()[-top_k:]
        )


class DEPENDENCY:
    def __init__(self, tag, depend):
        self._tag = tag
        self._depend = depend

    def predict(self, string):
        """
        Tag a string

        Parameters
        ----------
        string : str

        Returns
        -------
        string: tagged string
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        string = entities_textcleaning(string)
        if len(string) > 120:
            raise Exception(
                'Dependency parsing only able to accept string less than 120 words'
            )
        batch_x = [features_crf(string, index) for index in range(len(string))]
        tagging = self._tag.predict_single(batch_x)
        batch_x = [
            features_crf_dependency(string, tagging, index)
            for index in range(len(string))
        ]
        depend = [int(i) for i in self._depend.predict_single(batch_x)]
        for i in range(len(depend)):
            if depend[i] == 0 and tagging[i] != 'root':
                tagging[i] = 'UNK'
            elif depend[i] != 0 and tagging[i] == 'root':
                tagging[i] = 'UNK'
            elif depend[i] > len(tagging):
                depend[i] = len(tagging)
        return (
            [(string[i], tagging[i]) for i in range(len(depend))],
            [(string[i], depend[i]) for i in range(len(depend))],
        )

    def print_features(self, top_k = 10):
        """
        Print important top-k features for tagging dependency

        Parameters
        ----------
        top_k : int
        """
        if not isinstance(top_k, int):
            raise ValueError('input must be an integer')
        print('Top-%d tagging positive:' % (top_k))
        state_features(Counter(self._tag.state_features_).most_common(top_k))

        print('\nTop-%d tagging negative:' % (top_k))
        state_features(
            Counter(self._tag.state_features_).most_common()[-top_k:]
        )

    def print_transitions_tag(self, top_k = 10):
        """
        Print important top-k transitions for tagging dependency

        Parameters
        ----------
        top_k : int
        """
        if not isinstance(top_k, int):
            raise ValueError('input must be an integer')
        print('Top-%d likely tagging transitions:' % (top_k))
        transitions(Counter(self._tag.transition_features_).most_common(top_k))

        print('\nTop-%d unlikely tagging transitions:' % (top_k))
        transitions(
            Counter(self._tag.transition_features_).most_common()[-top_k:]
        )

    def print_transitions_index(self, top_k = 10):
        """
        Print important top-k transitions for indexing dependency

        Parameters
        ----------
        top_k : int
        """
        if not isinstance(top_k, int):
            raise ValueError('input must be an integer')
        print('Top-%d likely indexing transitions:' % (top_k))
        transitions(
            Counter(self._depend.transition_features_).most_common(top_k)
        )

        print('\nTop-%d unlikely indexing transitions:' % (top_k))
        transitions(
            Counter(self._depend.transition_features_).most_common()[-top_k:]
        )


class USER_XGB:
    def __init__(self, xgb, label, vectorize, cleaning = simple_textcleaning):
        self.xgb = xgb
        self.label = label
        self.vectorize = vectorize
        self._cleaning = cleaning

    def predict(self, string, get_proba = False):
        """
        Classify a string

        Parameters
        ----------
        string : str
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: result
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        vectors = self.vectorize.transform([self._cleaning(string)])
        result = self.xgb.predict(
            xgb.DMatrix(vectors), ntree_limit = self.xgb.best_ntree_limit
        )[0]
        if get_proba:
            return {self.label[i]: result[i] for i in range(len(result))}
        else:
            return self.label[np.argmax(result)]

    def predict_batch(self, strings, get_proba = False):
        """
        Classify a list of strings

        Parameters
        ----------
        strings: list
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: list of results
        """
        if not isinstance(strings, list):
            raise ValueError('input must be a list')
        if not isinstance(strings[0], str):
            raise ValueError('input must be list of strings')
        strings = [self._cleaning(string) for string in strings]
        vectors = self.vectorize.transform(strings)
        results = self.xgb.predict(
            xgb.DMatrix(vectors), ntree_limit = self.xgb.best_ntree_limit
        )
        if get_proba:
            outputs = []
            for result in results:
                outputs.append(
                    {self.label[i]: result[i] for i in range(len(result))}
                )
            return outputs
        else:
            return [self.label[i] for i in np.argmax(results, axis = 1)]


class USER_BAYES:
    def __init__(
        self, multinomial, label, vectorize, cleaning = simple_textcleaning
    ):
        self.multinomial = multinomial
        self.label = label
        self.vectorize = vectorize
        self._cleaning = cleaning

    def predict(self, string, get_proba = False):
        """
        Classify a string

        Parameters
        ----------
        string : str
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: result
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        vectors = self.vectorize.transform([self._cleaning(string)])
        if get_proba:
            result = self.multinomial.predict_proba(vectors)[0]
            return {self.label[i]: result[i] for i in range(len(result))}
        else:
            return self.label[self.multinomial.predict(vectors)[0]]

    def predict_batch(self, strings, get_proba = False):
        """
        Classify a list of strings

        Parameters
        ----------
        strings: list
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: list of results
        """
        if not isinstance(strings, list):
            raise ValueError('input must be a list')
        if not isinstance(strings[0], str):
            raise ValueError('input must be list of strings')
        strings = [self._cleaning(string) for string in strings]
        vectors = self.vectorize.transform(strings)
        if get_proba:
            results = self.multinomial.predict_proba(vectors)
            outputs = []
            for result in results:
                outputs.append(
                    {self.label[i]: result[i] for i in range(len(result))}
                )
            return outputs
        else:
            return [
                self.label[result]
                for result in self.multinomial.predict(vectors)
            ]


class TOXIC:
    def __init__(self, models, vectors):
        self._models = models
        self._vectors = vectors
        self._class_names = [
            'toxic',
            'severe_toxic',
            'obscene',
            'threat',
            'insult',
            'identity_hate',
        ]

    def _stack(self, strings):
        char_features = self._vectors['char'].transform(strings)
        word_features = self._vectors['word'].transform(strings)
        return hstack([char_features, word_features])

    def predict(self, string, get_proba = False):
        """
        Classify a string

        Parameters
        ----------
        string : str
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: result
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        stacked = self._stack([classification_textcleaning(string, True)])
        result = {} if get_proba else []
        for no, label in enumerate(self._class_names):
            if get_proba:
                result[label] = self._models[no].predict_proba(stacked)[0, 1]
            else:
                prob = self._models[no].predict(stacked)[0]
                if prob:
                    result.append(label)
        return result

    def predict_batch(self, strings, get_proba = False):
        """
        Classify a list of strings

        Parameters
        ----------
        strings: list
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: list of results
        """
        if not isinstance(strings, list):
            raise ValueError('input must be a list')
        if not isinstance(strings[0], str):
            raise ValueError('input must be list of strings')
        stacked = self._stack(
            [classification_textcleaning(i, True) for i in strings]
        )
        result = []
        for no in range(len(self._class_names)):
            if get_proba:
                probs = self._models[no].predict_proba(stacked)[:, 1]
            else:
                probs = self._models[no].predict(stacked)
            result.append(probs)
        result = np.array(result).T
        dicts = []
        for row in result:
            nested = {} if get_proba else []
            for no, label in enumerate(self._class_names):
                if get_proba:
                    nested[label] = row[no]
                else:
                    if row[no]:
                        nested.append(label)
            dicts.append(nested)
        return dicts


class LANGUAGE_DETECTION:
    def __init__(self, model, label, vectorizer, mode = 'sklearn'):
        self._model = model
        self._label = label
        self._vectorizer = vectorizer
        self._mode = mode

    def predict(self, string, get_proba = False):
        """
        Classify a string

        Parameters
        ----------
        string : str
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: result
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        string = language_detection_textcleaning(string)
        vectors = self._vectorizer.transform([string])
        if self._mode == 'xgb':
            result = self._model.predict(
                xgb.DMatrix(vectors), ntree_limit = self._model.best_ntree_limit
            )[0]
            if get_proba:
                return {self._label[i]: result[i] for i in range(len(result))}
            else:
                return self._label[np.argmax(result)]
        else:
            if get_proba:
                result = self._model.predict_proba(vectors)[0]
                return {self._label[i]: result[i] for i in range(len(result))}
            else:
                return self._label[self._model.predict(vectors)[0]]

    def predict_batch(self, strings, get_proba = False):
        """
        Classify a list of strings

        Parameters
        ----------
        strings: list
        get_proba: bool, optional (default=False)
            If True, it will return probability of classes.

        Returns
        -------
        string: list of results
        """
        if not isinstance(strings, list):
            raise ValueError('input must be a list')
        if not isinstance(strings[0], str):
            raise ValueError('input must be list of strings')
        strings = [
            language_detection_textcleaning(string) for string in strings
        ]
        vectors = self._vectorizer.transform(strings)

        if self._mode == 'xgb':
            results = self._model.predict(
                xgb.DMatrix(vectors), ntree_limit = self._model.best_ntree_limit
            )
            if get_proba:
                outputs = []
                for result in results:
                    outputs.append(
                        {self._label[i]: result[i] for i in range(len(result))}
                    )
                return outputs
            else:
                return [self._label[i] for i in np.argmax(results, axis = 1)]
        else:
            if get_proba:
                results = self._model.predict_proba(vectors)
                outputs = []
                for result in results:
                    outputs.append(
                        {self._label[i]: result[i] for i in range(len(result))}
                    )
                return outputs
            else:
                return [
                    self._label[result]
                    for result in self._model.predict(vectors)
                ]



import sys
import warnings

if not sys.warnoptions:
    warnings.simplefilter('ignore')

import numpy as np
import json
from fuzzywuzzy import fuzz
from unidecode import unidecode
from collections import Counter
from ._utils._utils import (
    load_graph,
    check_file,
    check_available,
    generate_session,
)
from .texts._text_functions import (
    normalizer_textcleaning,
    stemmer_str_idx,
    pad_sentence_batch,
    ENGLISH_WORDS,
)
from .texts._tatabahasa import (
    rules_normalizer,
    consonants,
    vowels,
    sounds,
    GO,
    PAD,
    EOS,
    UNK,
)
from .spell import _return_possible, _edit_normalizer, _return_known
from .similarity import is_location
from ._utils._paths import MALAY_TEXT, PATH_NORMALIZER, S3_PATH_NORMALIZER


class _DEEP_NORMALIZER:
    def __init__(self, x, logits, sess, dicts, corpus):
        self._sess = sess
        self._x = x
        self._logits = logits
        self._dicts = dicts
        self._dicts['rev_dictionary_to'] = {
            int(k): v for k, v in self._dicts['rev_dictionary_to'].items()
        }
        self.corpus = corpus

    def normalize(self, string, check_english = True):
        """
        Normalize a string.

        Parameters
        ----------
        string : str

        Returns
        -------
        string: normalized string
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        if not isinstance(check_english, bool):
            raise ValueError('check_english must be a boolean')

        token_strings = normalizer_textcleaning(string).split()
        results, need_to_normalize = [], []
        for word in token_strings:
            if word.istitle():
                results.append(word)
                continue
            if check_english:
                if word in ENGLISH_WORDS:
                    results.append(word)
                    continue
            if word[0] == 'x' and len(word) > 1:
                result_string = 'tak '
                word = word[1:]
            else:
                result_string = ''
            if word[-2:] == 'la':
                end_result_string = ' lah'
                word = word[:-2]
            elif word[-3:] == 'lah':
                end_result_string = ' lah'
                word = word[:-3]
            else:
                end_result_string = ''
            if word in sounds:
                results.append(result_string + sounds[word] + end_result_string)
                continue
            if word in rules_normalizer:
                results.append(
                    result_string + rules_normalizer[word] + end_result_string
                )
                continue
            if word in self.corpus:
                results.append(result_string + word + end_result_string)
                continue
            results.append('replace__me__')
            need_to_normalize.append(word)

        normalized = []
        if len(need_to_normalize):
            idx = stemmer_str_idx(
                need_to_normalize, self._dicts['dictionary_from']
            )
            predicted = self._sess.run(
                self._logits,
                feed_dict = {self._x: pad_sentence_batch(idx, PAD)[0]},
            )
            for word in predicted:
                normalized.append(
                    ''.join(
                        [
                            self._dicts['rev_dictionary_to'][c]
                            for c in word
                            if c not in [GO, PAD, EOS, UNK]
                        ]
                    )
                )
        cp_results, current_replace = [], 0
        for i in range(len(results)):
            if 'replace__me__' in results[i]:
                if current_replace < len(normalized):
                    results[i] = normalized[current_replace]
                    cp_results.append(results[i])
                    current_replace += 1
            else:
                cp_results.append(results[i])

        return ' '.join(cp_results)


class _SPELL_NORMALIZE:
    def __init__(self, corpus):
        self.corpus = Counter(corpus)

    def normalize(self, string, debug = True, check_english = True):
        """
        Normalize a string

        Parameters
        ----------
        string : str

        debug : bool, optional (default=True)
            If true, it will print character similarity distances.
        check_english: bool, (default=True)
            check a word in english dictionary

        Returns
        -------
        string: normalized string
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        if not isinstance(debug, bool):
            raise ValueError('debug must be a boolean')
        if not isinstance(check_english, bool):
            raise ValueError('check_english must be a boolean')

        result = []
        for word in normalizer_textcleaning(string).split():
            if word.istitle():
                result.append(word)
                continue
            if check_english:
                if word in ENGLISH_WORDS:
                    result.append(word)
                    continue
            if len(word) > 2:
                if word[-2] in consonants and word[-1] == 'e':
                    word = word[:-1] + 'a'
            if word[0] == 'x' and len(word) > 1:
                result_string = 'tak '
                word = word[1:]
            else:
                result_string = ''
            if word[-2:] == 'la':
                end_result_string = ' lah'
                word = word[:-2]
            elif word[-3:] == 'lah':
                end_result_string = ' lah'
                word = word[:-3]
            else:
                end_result_string = ''
            if word in sounds:
                result.append(result_string + sounds[word] + end_result_string)
                continue
            if word in rules_normalizer:
                result.append(
                    result_string + rules_normalizer[word] + end_result_string
                )
                continue
            if word in self.corpus:
                result.append(result_string + word + end_result_string)
                continue
            candidates = (
                _return_known([word], self.corpus)
                or _return_known(_edit_normalizer(word), self.corpus)
                or _return_possible(word, self.corpus, _edit_normalizer)
                or [word]
            )
            candidates = list(candidates)
            candidates = [
                (candidate, is_location(candidate))
                for candidate in list(candidates)
            ]
            if debug:
                print([(k, fuzz.ratio(word, k[0])) for k in candidates], '\n')
            strings = [fuzz.ratio(word, k[0]) for k in candidates]
            descending_sort = np.argsort(strings)[::-1]
            selected = None
            for index in descending_sort:
                if not candidates[index][1]:
                    selected = candidates[index][0]
                    break
            selected = (
                candidates[descending_sort[0]][0] if not selected else selected
            )
            result.append(result_string + selected + end_result_string)
        return ' '.join(result)


class _FUZZY_NORMALIZE:
    def __init__(self, normalized, corpus):
        self.normalized = normalized
        self.corpus = corpus

    def normalize(self, string, fuzzy_ratio = 70, check_english = True):
        """
        Normalize a string.

        Parameters
        ----------
        string : str
        fuzzy_ratio: int, (default=70)
            ratio of similar characters by positions, if 90, means 90%
        check_english: bool, (default=True)
            check a word in english dictionary

        Returns
        -------
        string: normalized string
        """
        if not isinstance(string, str):
            raise ValueError('input must be a string')
        if not isinstance(fuzzy_ratio, int):
            raise ValueError('fuzzy_ratio must be an integer')
        if not isinstance(check_english, bool):
            raise ValueError('check_english must be a boolean')

        result = []
        for word in normalizer_textcleaning(string).split():
            if word.istitle():
                result.append(word)
                continue
            if check_english:
                if word in ENGLISH_WORDS:
                    result.append(word)
                    continue
            if len(word) > 2:
                if word[-2] in consonants and word[-1] == 'e':
                    word = word[:-1] + 'a'
            if word[0] == 'x' and len(word) > 1:
                result_string = 'tak '
                word = word[1:]
            else:
                result_string = ''
            if word[-2:] == 'la':
                end_result_string = ' lah'
                word = word[:-2]
            elif word[-3:] == 'lah':
                end_result_string = ' lah'
                word = word[:-3]
            else:
                end_result_string = ''
            if word in sounds:
                result.append(result_string + sounds[word] + end_result_string)
                continue
            if word in rules_normalizer:
                result.append(
                    result_string + rules_normalizer[word] + end_result_string
                )
                continue
            if word in self.corpus:
                result.append(result_string + word + end_result_string)
                continue
            results = []
            for i in range(len(self.normalized)):
                results.append(
                    np.mean([fuzz.ratio(word, k) for k in self.normalized[i]])
                )
            if len(np.where(np.array(results) > fuzzy_ratio)[0]) < 1:
                result.append(result_string + word + end_result_string)
                continue
            result.append(
                result_string
                + self.corpus[np.argmax(results)]
                + end_result_string
            )
        return ' '.join(result)


def fuzzy(corpus):
    """
    Train a fuzzy logic Normalizer

    Parameters
    ----------
    corpus : list of strings. Prefer to feed with malaya.load_malay_dictionary()

    Returns
    -------
    FUZZY_NORMALIZE: Trained malaya.normalizer._FUZZY_NORMALIZE class
    """
    if not isinstance(corpus, list):
        raise ValueError('corpus must be a list')
    if not isinstance(corpus[0], str):
        raise ValueError('corpus must be list of strings')
    corpus = [unidecode(w) for w in corpus]
    transform = []
    for i in corpus:
        i = i.lower()
        result = []
        result.append(i)
        result.append(''.join(char for char in i if char not in vowels))
        if i[0] in consonants and i[-1] in consonants:
            inner = i[0] + i[-1]
            result.append(inner)
        if i[-1] == 'a':
            inner = i[:-1]
            result.append(i[:-1] + 'e')
            inner = i + 'k'
            result.append(inner)
        if i[1] in vowels and i[0] in consonants:
            inner = i[0] + i[2:]
            result.append(inner)
        if i[-2] in vowels and i[-1] in consonants:
            inner = i[:-2] + i[-1]
            result.append(inner)
        result.append(i[0] + i[-1])
        if i[-2:] == 'ar':
            result.append(i[:-2] + 'o')
        if i[:2] == 'ha':
            result.append(i[1:])
        result = filter(None, result)
        transform.append(list(set(result)))
    return _FUZZY_NORMALIZE(transform, corpus)


def spell(corpus):
    """
    Train a Spelling Normalizer

    Parameters
    ----------
    corpus : list of strings. Prefer to feed with malaya.load_malay_dictionary()

    Returns
    -------
    SPELL_NORMALIZE: Trained malaya.normalizer._SPELL_NORMALIZE class
    """
    if not isinstance(corpus, list):
        raise ValueError('corpus must be a list')
    if not isinstance(corpus[0], str):
        raise ValueError('corpus must be list of strings')
    return _SPELL_NORMALIZE([unidecode(w) for w in corpus])


def basic(string):
    """
    Use basic rules-based to normalize a string.

    Parameters
    ----------
    string: str

    Returns
    -------
    string: normalized string
    """
    if not isinstance(string, str):
        ValueError('input must be a string')
    result = []
    for word in normalizer_textcleaning(string).split():
        if word.istitle():
            result.append(word)
            continue
        if word in sounds:
            result.append(sounds[word])
        elif word[-1] == '2':
            result.append(word[:-1])
        else:
            result.append(word)
    return ' '.join(result)


def available_deep_model():
    """
    List available deep learning stemming models.
    """
    return ['lstm', 'bahdanau', 'luong']


def deep_model(corpus, model = 'bahdanau', validate = True):
    """
    Load deep-learning model to normalize a string.

    Parameters
    ----------
    validate: bool, optional (default=True)
        if True, malaya will check model availability and download if not available.

    Returns
    -------
    DEEP_NORMALIZER: malaya.normalizer._DEEP_NORMALIZER class

    """
    if not isinstance(corpus, list):
        raise ValueError('corpus must be a list')
    if not isinstance(corpus[0], str):
        raise ValueError('corpus must be list of strings')
    if validate:
        check_file(PATH_NORMALIZER[model], S3_PATH_NORMALIZER[model])
    else:
        if not check_available(PATH_NORMALIZER[model]):
            raise Exception(
                'normalizer is not available, please `validate = True`'
            )
    try:
        with open(PATH_NORMALIZER[model]['setting'], 'r') as fopen:
            dic_normalizer = json.load(fopen)
        g = load_graph(PATH_NORMALIZER[model]['model'])
    except:
        raise Exception(
            "model corrupted due to some reasons, please run malaya.clear_cache('normalizer/%s') and try again"
            % (model)
        )
    return _DEEP_NORMALIZER(
        g.get_tensor_by_name('import/Placeholder:0'),
        g.get_tensor_by_name('import/logits:0'),
        generate_session(graph = g),
        dic_normalizer,
        [unidecode(w) for w in corpus],
    )



import malaya

def test_naive_stemmer():
    makanan = malaya.naive_stemmer('makanan')
    perjalanan = malaya.naive_stemmer('perjalanan')
    assert makanan == 'makan' and perjalanan == 'jalan'

def test_sastrawi_stemmer():
    assert malaya.sastrawi_stemmer('menarik') == 'tarik'

def test_deep_stemmer():
    stemmer = malaya.deep_stemmer()
    assert stemmer.stem('saya sangat sukakan awak') == 'saya sangat suka awak'

def test_deep_stemmer_unknown():
    stemmer = malaya.deep_stemmer()
    assert len(stemmer.stem('!!!!!()*&^%!^@%4'))



topic = {
    'mahathir': [
        'tun mahathir',
        'madey',
        'dr mahathir',
        'tun m',
        'mahathir',
        'madir',
        'dr m',
        'mahathir muhamad',
    ],
    'anwar ibrahim': ['anwar ibrahim', 'anwar'],
    'najib razak': [
        'najib razak',
        'ajib',
        'pakjib',
        'pok jib',
        'pak jib',
        'najib',
        'tan sri najib',
    ],
    'pakatan harapan': ['pakatan harapan'],
    'syed saddiq': ['syed saddiq', 'syedsaddiq', 'saddiq'],
    'parti keadilan rakyat': ['parti keadilan rakyat', 'people justice party'],
    'umno': ['united malays', 'umno', 'united malays national organization'],
    'barisan nasional': ['barisan nasional'],
    'parti islam semalaysia': [
        'parti islam',
        'malaysian islamic party',
        'parti islam semalaysia',
        'parti islam malaysia',
    ],
    'nurul izzah': ['nurul izzah', 'izzah'],
    'Tunku Ismail Idris': ['tunku ismail', 'tunku ismail idris', 'tmj'],
    'pdrm': [
        'royal police malaysia',
        'polis diraja malaysia',
        'pdrm',
        'polis',
        'polis diraja',
    ],
    '1mdb': [
        '1mdb',
        '1malaysia development berhad',
        '1malaysia development',
        '1malaysia development bhd',
        'mdb',
    ],
    'makanan': [
        'kfc',
        'murtabak',
        'sate',
        'nasi',
        'nandos',
        'mcdonald',
        'mcd',
        'makanan',
        'martabak',
        'roti',
        'canai',
        'sushi king',
    ],
    'ganja': ['cannabis', 'heroin', 'weed', 'ganja'],
    'gst': ['gst', 'good service tax'],
    'malaysian chinese association': ['malaysian chinese association'],
    'democratic action party': ['democratic action party'],
    'amanah': ['national trust party', 'amanah', 'parti amanah negara'],
    'ppbm': [
        'parti pribumi bersatu malaysia',
        'ppbm',
        'malaysian united',
        'parti pribumi',
        'malaysian united indigenous party',
    ],
    'mic': [
        'malaysian indian',
        'malaysian indian congress',
        'indian congress',
        'mic',
    ],
    'kerajaan': ['gemen', 'kerajaan', 'kemen', 'gomen', 'government'],
    'pembangkang': ['opposition', 'pembangkang'],
    'tiga penjuru': ['tiga penjuru'],
    'Pilihan Raya Umum Ke-14': [
        'election',
        'ge14',
        'pilihan raya',
        'pilihan raya umum ke-14',
        'pru14',
    ],
    'Pusat Daerah Mangundi': ['pdm', 'pusat daerah mangundi', 'pusat mengundi'],
    'Tun Daim Zainuddin': [
        'daim zainuddin',
        'daim',
        'tun daim',
        'tun daim zainuddin',
    ],
    'Datuk Seri Abdul Hadi Awang': [
        'hadi',
        'datuk seri abdul hadi awang',
        'hadi awang',
    ],
    'Majlis Pakatan Harapan': ['majlis pakatan harapan'],
    'Wan Azizah': ['kak wan', 'azizah', 'wan azizah'],
    'Parti Pribumi Bersatu Malaysia': [
        'parti pribumi bersatu malaysia',
        'ppbm',
        'parti bersatu',
        'parti pribumi',
    ],
    'Datuk Seri Azmin Ali': ['azmin ali', 'datuk seri azmin ali', 'azmin'],
    'masalah air': ['air', 'masalah air', 'isu air'],
    'orang asli': [
        'perkampungan asli',
        'perkampungan orang asli',
        'orang asli',
        'tenau',
    ],
    'Datuk Johari Abdul': ['johari', 'johari abdul', 'datuk johari abdul'],
    'Tengku Razaleigh Hamzah': ['tengku razaleigh hamzah', 'razaleigh'],
    'Tan Sri Dr Rais Yatim': ['rais yatim', 'tan sri dr rais yatim'],
    'kaum cina': ['cina', 'chinese', 'kaum cina'],
    'k-pop': ['korea', 'k-pop'],
    'kaum india': ['kaum india', 'india', 'indian'],
    'rafidah aziz': ['rafidah', 'rafidah aziz'],
    'rumah mampu milik': ['rmm', 'mampu milik', 'rmmj', 'rumah mampu milik'],
    'universiti teknologi mara': [
        'universiti teknologi mara',
        'teknologi mara',
        'uitm',
    ],
    'universiti teknologi malaysia': ['utm', 'universiti teknologi malaysia'],
    'universiti malaya': ['universiti malaya'],
    'pendidikan': [
        'peruntukan',
        'pendidikan',
        'sekolah',
        'tvet',
        'untuk semua',
        'universiti',
        'asean',
        'prasekolah',
        'vokasional',
        'ilmu',
    ],
    'sekolah': ['sekolah'],
    'universiti': ['universiti'],
    'maktab rendah sains mara': ['maktab rendah sains mara'],
    'pos laju': ['pos laju', 'poslaju'],
    'hari raya': ['aidiladha', 'aidilfitri', 'hari raya'],
    'kesihatan': [
        'denggi',
        'klinik',
        'fiber',
        'kencing manis',
        'diet',
        'kanser',
        'vitamin',
        'pusat jantung',
        'kesihatan',
        'hospital',
    ],
    'kewangan': ['hutang negara', 'kewangan', 'ekonomi negara', 'bank'],
    'hutang negara': [
        'national debt',
        'malaysian debt',
        'hutang negara',
        'malaysia debt',
    ],
    'ekonomi': [
        'asia tenggara',
        'dftz',
        'pelabur',
        'ekonomi',
        'dagang',
        'modal teroka',
        'swasta',
        'magic',
        'cradle',
        'stvp',
        'map',
        'keusahawan',
        'bebas digital',
        'global',
        'ekonomi digital',
        'mycreative',
    ],
    'sosial': [
        'sosial',
        'kehidupan',
        'taraf hidup',
        'sosiologi',
        'keusahawan',
        'masyarakat',
    ],
    'politik': [
        'pru',
        'transformasi',
        'exco',
        'menteri besar',
        'kabinet',
        'jawatan',
        'debat',
        'menteri',
        'politik',
        'senator',
        'parti',
        'pemimpin',
        'undi luar',
        'calon speaker',
    ],
    'startup companies': [
        'pelabur',
        'cradle',
        'magic',
        'startup companies',
        'keusahawan sosial',
        'startup',
        'ekonomi digital',
        'mycreative',
    ],
    'felda': [
        'lembaga kemajuan tanah persekutuan',
        'federal land development authority',
        'felda global ventures',
        'felda',
    ],
    'kwsp': ['epf', 'employees provident fund', 'kwsp', 'kumpulan wang'],
    'bank malaysia': [
        'ocbc',
        'cimb',
        'citibank',
        'hsbc',
        'affin bank',
        'alliance bank',
        'bangkok bank',
        'hong leong bank',
        'ambank',
        'bank malaysia',
        'rhb',
    ],
    'bank negara': ['bank negara'],
    'perdana menteri': ['perdana menteri', 'prime minister'],
    'timbalan perdana menteri': [
        'deputy prime minister',
        'timbalan perdana menteri',
    ],
    'menteri dalam negeri': [
        'ministry of home affairs',
        'menteri dalam negeri',
    ],
    'menteri kewangan': ['menteri kewangan', 'ministry of finance', 'mof'],
    'menteri pertahanan': ['menteri pertahanan', 'minister of defence'],
    'majlis penasihat': [
        'tun daim',
        'zeti aziz',
        'council of elders',
        'ks jomo',
        'robert kuok',
        'majlis penasihat',
        'hassan merican',
    ],
    'skim peduli sihat': ['skim peduli sihat'],
    'ptptn': [
        'ptptn',
        'perbadanan tabung pendidikan tinggi nasional',
        'tabung pendidikan',
    ],
    'projek mega': ['projek mega'],
    'gaji minimum': ['gaji minimum'],
    'menyiasat skandal': [
        'korupsi kerajaan',
        'skandal bn',
        'skandal kerajaan',
        'government scandal',
        'menyiasat skandal',
    ],
    'highway tol': [
        'highway tol',
        'tol',
        'hapuskan tol',
        'harga tol',
        'kutipan tol',
    ],
    'tabung haji': ['tabung haji'],
    'tentera malaysia': [
        'tentera',
        'malaysian armed forces',
        'tentera udara',
        'tentera malaysia',
        'tentera darat',
        'tentera laut',
    ],
    'infrastruktur': [
        'luar bandar',
        'pan borneo',
        'infrastruktur',
        'lebuhraya pan borneo',
        'transformasi luar bandar',
        'belb',
        'bekalan elektrik',
        'bekalan air',
    ],
    'kos sara hidup': [
        'bantuan rakyat',
        'k1m',
        'brim',
        'kos sara hidup',
        'br1m',
        'bantuan pelajar',
    ],
    'pengangkutan awam': ['lrt', 'terminal', 'pengangkutan awam', 'bas', 'mrt'],
    'perkhidmatan awam': [
        'dasar pengambilan',
        'perkhidmatan awam',
        'oku',
        'peluang pekerjaan',
        'utc',
        'rtc',
    ],
    'wanita': ['wanita', 'talentcorp', 'dewi', 'kembali bekerja'],
    'rafizi ramli': ['rafizi', 'rafizi ramli'],
    'bersatu': ['bersatu'],
    'bernama': ['bernama'],
    'rela': ['rela', 'anggota jabatan sukarelawan malaysia'],
    'kementerian dalam negeri': ['kementerian dalam negeri', 'kdn'],
    'donald trump': ['trump', 'donald', 'donald trump'],
    'kaum melayu': ['malay', 'melayu', 'kaum melayu'],
    'Survei Institut Darul Ehsan': [
        'survei institut',
        'survei institut darul ehsan',
    ],
    'Inisiatif Peduli Rakyat': [
        'peduli rakyat',
        'ipr',
        'inisiatif peduli rakyat',
    ],
    'astro awani': ['astro', 'astro awani', 'awani'],
    'Datuk Seri Ti Lian Ker': ['ti lian', 'datuk seri ti lian ker', 'lian ker'],
    'Nga Kor Ming': ['kor ming', 'nga kor ming'],
    'Lee Kuan Yew': ['kuan yew', 'lee kuan yew'],
    'rosmah mansur': ['rosmah', 'rosmah mansur'],
    'facebook': ['fb', 'facebook'],
    '1malaysia': ['1malaysia'],
    'petronas': ['petronas'],
    'keluar parti': [
        'pemecatan ahli',
        'khianati parti',
        'khianat parti',
        'keluar parti',
    ],
    'sabotaj parti': ['sabotaj calon', 'sabotaj parti'],
    'kotak undi': ['spr', 'kotak undi', 'salah kotak'],
    'perkasa': ['pertubuhan pribumi perkasa', 'pertubuhan pribumi', 'perkasa'],
    'Tan Sri Mokhzani Mahathir': [
        'tan sri mokhzani mahathir',
        'mokhzani',
        'mokhzani mahathir',
    ],
    'teknologi': [
        'smartphone',
        'machine learning',
        'fiber optic',
        'telefon pintar',
        'big data',
        'kecerdasan buatan',
        'teknologi',
        'internet',
        'artificial intelligence',
        'iot',
        'drone',
    ],
    'internet': ['broadband', 'fiber optic', '4g', '3g', 'internet'],
    'artificial intelligence': [
        'artificial intelligence',
        'bayes',
        'machine learning',
    ],
    'humanoid': ['humanoid', 'robot', 'sofia humanoid'],
    'Rais Yatim': ['rais yatim', 'rais'],
    'Anthony Loke Siew Fook': ['anthony loke siew fook', 'loke siew'],
    'Ahli Dewan Undangan Negeri': [
        'ahli dewan undangan negeri',
        'ahli dewan undangan',
        'adun',
    ],
    '#fakenews': ['#fakenews', 'fake news', 'berita palsu'],
    'suruhanjaya pilihan raya malaysia': [
        'suruhanjaya pilihan raya',
        'spr',
        'suruhanjaya pilihan raya malaysia',
        'sprm',
    ],
    'kertas undi': ['kertas undi'],
    'KKMM': [
        'kementerian komunikasi multimedia',
        'kkmm',
        'kementerian komunikasi',
        'kementerian komunikasi dan multimedia',
    ],
    'akta pilihan raya': ['akta pilihan raya'],
    'undi pos': ['undian pos', 'pengundi pos', 'undi pos'],
    'undi rosak': ['undi rosak', '#undirosak'],
    'harga minyak': [
        'ron95',
        'harga minyak',
        'oil',
        'petrol',
        'diesel',
        'minyak',
        'ron97',
    ],
    'petrol': ['ron95', 'ron', 'petrol', 'ron97'],
    'government subsidies': ['government subsidies', 'subsidies'],
    'akademik': [
        'akademik',
        'phd',
        'ijazah',
        'sarjana',
        'degree',
        'diploma',
        'master',
    ],
    'mh370': ['mh370'],
    'kemelangan penumpang cedera': [
        'penumpang cedera',
        'kemelangan penumpang cedera',
    ],
    'kemelangan maut': ['kemelangan maut', 'maut'],
    'Recep Tayyip Erdogan': ['recep tayyip', 'recep tayyip erdogan', 'erdogan'],
    'individu penjara': ['penjara', 'individu penjara'],
    'kes rogol': ['kes rogol', 'rogol'],
    'isytihar darurat': ['isytihar darurat', 'darurat'],
    'cambridge analytica': [
        'analytica',
        'strategic communication laboratories',
        'cambridge analytica',
    ],
    'kokain': ['kokain', 'cocaine'],
    'bebas tahanan': ['tahanan', 'tahanan bebas', 'bebas tahanan'],
    'sosial media': ['fb', 'instragram', 'twitter', 'sosial media', 'facebook'],
    'twitter': ['twitter'],
    'instagram': ['instagram'],
    'mati dipukul': ['mati dipukul'],
    'pengedar dadah': ['dadah', 'pengedar dadah', 'pengedar'],
    'kematian wabak': ['ebola', 'penyakit', 'wabak', 'h1n1', 'kematian wabak'],
    'letupan bom': ['letupan bom', 'pengebom', 'bom'],
    'isu dadah': ['dadah', 'isu dadah'],
    'diesel': ['diesel'],
    'china': ['china'],
    'saudi arabia': ['arab', 'saudi arabia', 'saudi'],
    'gaji menteri': ['gaji menteri'],
    'jabatan bubar': ['jabatan bubar', 'dibubarkan', 'bubar'],
    'harakah': ['harakah'],
    'apple product': [
        'apple product',
        'macintosh',
        'airpod',
        'iphone',
        'ipad',
        'imac',
        'macbook',
        'apple',
    ],
    'iphone': ['iphone'],
    'samsung smartphone': [
        'samsung smartphone',
        'samsung galaxy',
        'samsung note',
    ],
    'huawei smartphone': [
        'huawei mate',
        'huawei nova',
        'huawei smartphone',
        'huawei honor',
        'huawei p',
    ],
    'xiaomi smartphone': ['xiaomi mi', 'xiaomi redmi', 'xiaomi smartphone'],
    'asus smartphone': ['asus smartphone', 'zenfone'],
    'oppo smartphone': ['oppo smartphone', 'oppo r', 'oppo f', 'oppo a'],
    'lenovo smartphone': [
        'lenovo phab',
        'lenovo smartphone',
        'lenovo vibe',
        'lenovo a',
    ],
    'vivo smartphone': ['vivo y', 'vivo smartphone', 'vivo v'],
    'telekom malaysia': [
        'telekom malaysia',
        'unifi',
        'streamyx',
        'telekom',
        'tm net',
        'axiata',
    ],
    'unifi': ['unifi'],
    'robert kuok': ['robert kuok', 'kuok'],
    'macc': ['macc'],
    'piala thomas': ['piala thomas'],
    'fifa': ['world cup', 'piala dunia', 'fifa'],
    'programming language': [
        'ruby',
        'programming language',
        'c++',
        'java',
        'python',
    ],
    'agama': [
        'lesbian',
        'muslim',
        'lgbt',
        'free thinker',
        'kafir',
        'agama',
        'kristian',
        'islam',
        'gay',
        'buddha',
    ],
    'lgbt': [
        'lesbian',
        'asexual',
        'lgbt',
        'gay',
        'transgender',
        'bisexual',
        'biseksual',
    ],
    'islam': ['muslim', 'kafir', 'hadis', 'syiah', 'islam', 'wahabi'],
    'masyarakat': [
        'free thinker',
        'konservatif',
        'komunis',
        'liberalisme',
        'sosialis',
        'masyarakat',
    ],
    'idealogi': [
        'idealogi',
        'konservatif',
        'liberal',
        'komunis',
        'kapitalis',
        'sosialis',
    ],
    'parlimen': ['parlimen'],
    'Pusat Transformasi Bandar': [
        'pusat transformasi bandar',
        'utc',
        'pusat transformasi',
    ],
    'gerakan': ['gerakan', 'gerakan rakyat malaysia', 'parti gerakan'],
    'institut diraja': [
        'sultan',
        'institut diraja',
        'diraja',
        'institusi diraja',
        'kerabat',
    ],
    'tsunami fitnah': [
        'bohong',
        'tsunami politik',
        'janji palsu',
        'tiada modal',
        'tsunami',
        'tsunami fitnah',
        'fitnah',
        'jual cerita',
    ],
    'makro-ekonomi': [
        'macroeconomies',
        'makroekonomi',
        'macroeconomy',
        'makro-ekonomi',
    ],
    'mikro-ekonomi': [
        'microeconomy',
        'mikroekonomi',
        'mikro-ekonomi',
        'microeconomies',
    ],
    'stock market malaysia': [
        'macroeconomic impact',
        'usdmyr',
        'stock market malaysia',
        'economic policies',
        'stock votality',
        'subsidiaries',
        'national stock',
        'bloomberg',
        'foreign investor',
        'malaysia stock market',
        'malaysia revenue',
    ],
    'government revenue': [
        'stock market',
        'government revenue',
        'credit negative',
        'subsidies',
        'fuel subsidies',
        'foreign investor',
        'tax',
        'fiscal position',
        'moody',
    ],
    'ringgit fall': [
        'power transition',
        'ringgit',
        'ekonomi',
        'malaysian economy',
        'oil related',
        'subsidies',
        'ringgit fall',
        'foreign investor',
        'shock vote',
        'tax',
        'revenue',
        'moody',
    ],
    'bloomberg': ['bloomberg'],
    'new straits times': [
        'pro-government',
        'new straits times',
        'english newspaper',
        'straits times',
        'right wing',
        'media prima',
    ],
    'the star': ['the star'],
    'malay mail': ['malay mail'],
    'median salary': [
        'monthly salary',
        'median salary',
        'department of statistics',
        'per anum',
        'overtime',
        'employee',
        'wage',
        'allowance',
    ],
    'bursa malaysia': ['bursa malaysia', 'bursa'],
    'bumiputra': ['bumi putra', 'bumiputra', 'melayu'],
    'universiti islam antarabangsa': ['universiti islam antarabangsa'],
    'baling botol': ['baling botol'],
    'child marriage': ['child marriage', 'kahwin bawah umur', 'kahwin budak'],
    'bridge city park': ['bridge city park', 'citi park'],
    'malaysia baru': ['malaysia baharu', 'malaysia baru'],
    'produk berbahaya': ['produk berbahaya'],
    'keluar parlimen': ['keluar parlimen'],
    'musim durian': ['durian', 'musim durian'],
    'world cup': ['world cup', 'fifa'],
    'rtm': ['radio televisyen malaysia', 'rtm'],
    'motogp': ['motogp'],
    'euro 2020': ['euro 2020'],
    'ask me a question': ['ask me a question'],
    'thai cave': ['thai cave'],
    'dewan rakyat': ['dewan rakyat', 'dato ariff', 'parliament house'],
    'tabung harapan': ['tabung harapan'],
    'racist': ['racist', 'perkauman'],
    'singapura': ['singapura', 'singapore'],
    'sst': ['sales and service tax', 'sales service tax', 'sst'],
    'malaysia-indonesia': ['malaysia-indonesia'],
    'gaza': ['gaza'],
    'bola sepak': ['bola sepak'],
    'sepak takraw': ['sepak takraw'],
    'isu sosma': ['isu sosma', 'sosma'],
    'isu ecrl': ['isu ecrl', 'ecrl'],
    'royalti minyak': ['royalti minyak'],
    'Jabatan Agama Islam Wilayah Persekutuan': [
        'jawi',
        'jabatan agama islam wilayah persekutuan',
    ],
    'sinar harian': ['sinar harian'],
    'reformasi': ['reformasi'],
    'kes rasuah': ['kes rasuah', 'rasuah'],
    'kewangan dan perniagaan': [
        'kewangan',
        'niaga',
        'perniagaan',
        'duit',
        'kewangan dan perniagaan',
        'wang',
    ],
    'saham dan komoditi': ['wang', 'saham', 'saham dan komoditi', 'komoditi'],
    'isu kerugian': [
        'mata wang jatuh',
        'isu kerugian',
        'saham jatuh',
        'kerugian',
    ],
    'isu bumiputera': [
        'melayu',
        'isu bumiputera',
        'bumiputera',
        'cina',
        'india',
    ],
    'alam sekitar': ['alam sekitar', 'hidupan liar', 'biodiversiti', 'hutan'],
    'isu kemiskinan': [
        'duit takde',
        'kemiskinan',
        'takda duit',
        'takde duit',
        'isu kemiskinan',
        'miskin',
        'duit takda',
    ],
    'sumber asli': [
        'emas',
        'sumber asli',
        'getah',
        'sawit',
        'timah',
        'perak',
        'besi',
        'kuprum',
    ],
}



# Malaya Natural Language Toolkit
#
# Copyright (C) 2019 Malaya Project
# Licensed under the MIT License
# Author: huseinzol05 <husein.zol05@gmail.com>
# URL: <https://malaya.readthedocs.io/>
# For license information, see https://github.com/huseinzol05/Malaya/blob/master/LICENSE

from .topic import topic
from .calon import calon
from .influencer import influencer
from .location import location
from .shortform import shortform
